@@ -9,13 +9,13 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: e3d_bv.erl 206 2008-01-04 16:08:58Z bjorng $
+%%     $Id$
 %%
 
 -module(e3d_bv).
 -export([eigen_vecs/1,quickhull/1,covariance_matrix/1]).
 
--import(e3d_vec, [dot/2,add/2,sub/2,average/1,norm/1,normal/1]).
+-import(e3d_vec, [dot/2,add/2,average/1,normal/1]).
 -import(lists, [foldl/3]).
 
 -compile(inline).
