@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: e3d_obj.erl 285 2008-03-09 11:54:16Z bjorng $
+%%     $Id$
 %%
 
 -module(e3d_obj).
@@ -17,8 +17,7 @@
 -include("e3d.hrl").
 -include("e3d_image.hrl").
 
--import(lists, [reverse/1,reverse/2,sort/1,keysearch/3,foreach/2,
-		map/2,foldl/3]).
+-import(lists, [reverse/1,reverse/2,foreach/2,foldl/3]).
 
 -record(ost,
 	{v=[],					%Vertices.
@@ -26,6 +25,7 @@
 	 vn=[],					%Vertex normals.
 	 f=[],					%Faces.
 	 g=[],					%Groups.
+	 s=[],					%Smooth groups.
 	 mat=[],				%Current material.
 	 matdef=[],				%Material definitions.
 	 dir,					%Directory of .obj file.
@@ -58,14 +58,15 @@ import(Name) ->
 import_1(Fd, Dir) ->
     Ost0 = read(fun parse/2, Fd, #ost{dir=Dir}),
     Ost = remember_eof(Ost0),
-    #ost{v=Vtab0,vt=TxTab0,f=Ftab0,g=Gs0,vn=VnTab0,matdef=Mat} = Ost,
+    #ost{v=Vtab0,vt=TxTab0,f=Ftab0,g=Gs0,vn=VnTab0,matdef=Mat,s=S0} = Ost,
     Vtab = reverse(Vtab0),
     TxTab = reverse(TxTab0),
     VnTab = reverse(VnTab0),
     Ftab = make_ftab(Ftab0, []),
     Gs1 = reverse(Gs0),
     Gs = separate(Gs1, []),
-    Template = #e3d_mesh{type=polygon,vs=Vtab,tx=TxTab,ns=VnTab},
+    He = hard_edges(S0, Ftab),
+    Template = #e3d_mesh{type=polygon,vs=Vtab,tx=TxTab,ns=VnTab,he=He},
     Objs = make_objects(Gs, Ftab, Template),
     #e3d_file{objs=Objs,mat=Mat}.
 
@@ -81,14 +82,70 @@ separate([{name,Name,Start}|T0], Acc) ->
 separate_1([{Name,S,E}|T], E, Acc) ->
     separate_1(T, S, [{Name,E-S}|Acc]);
 separate_1([], _, Acc) -> Acc.
+
+hard_edges(S0, Ftab) ->
+    S = smooth_groups(S0, length(Ftab)),
+    hard_edges_1(Ftab, S, []).
+
+hard_edges_1([#e3d_face{vs=[]}|Fs], [_|Sgs], Acc) ->
+    %% Ignore face with no vertices.
+    hard_edges_1(Fs, Sgs, Acc);
+hard_edges_1([#e3d_face{vs=Vs}|Fs], [Sg|Sgs], Acc0) ->
+    Acc = add_edges(Vs, hd(Vs), Sg, Acc0),
+    hard_edges_1(Fs, Sgs, Acc);
+hard_edges_1([], [], Acc) ->
+    R = sofs:relation(Acc),
+    F0 = sofs:relation_to_family(R),
+    F = sofs:to_external(F0),
+    foldl(fun({Edge,Sgs0}, He) ->
+		  %% Group 0 is special. Edges surrounded by faces with
+		  %% group zero will always be hard. Otherwise the rule
+		  %% is that the edge will be soft if all smooth groups
+		  %% are the same, and hard if they are different.
+		  case lists:usort(Sgs0) of
+		      [0] -> [Edge|He];
+		      [_] -> He;
+		      [_,_|_] -> [Edge|He]
+		  end
+	  end, [], F).
+
+add_edges([Va|[Vb|_]=T], Last, Sg, Acc) ->
+    add_edges(T, Last, Sg, [edge(Va, Vb, Sg)|Acc]);
+add_edges([Va], Vb, Sg, Acc) ->
+    [edge(Va, Vb, Sg)|Acc].
+    
+edge(A, B, Sg) when A < B -> {{A,B},Sg};
+edge(A, B, Sg) -> {{B,A},Sg}.
+
+%% smooth_groups([{Group,FaceNum}], NumFaces) -> [Group].
+%%  Collect smooth groups for the file. The result is a
+%%  list with the smooth group for each face. Faces not
+%%  explicitly given a smooth group in the file will be
+%%  be assigned group -1 (i.e. if no smooth groups are given,
+%%  all edges will be soft).
+
+smooth_groups(S0, NumFaces) ->
+    S = reverse(S0, [{eof,NumFaces}]),
+    smooth_groups_1(S, 0, -1, []).
+
+smooth_groups_1([{eof,F}], F, _, Acc) ->
+    reverse(Acc);
+smooth_groups_1([{G,F}|T], F, _, Acc) ->
+    smooth_groups_1(T, F+1, G, [G|Acc]);
+smooth_groups_1([_|_]=T, F, G, Acc) ->
+    smooth_groups_1(T, F+1, G, [G|Acc]).
     
 get_face_num([{eof,N}|_]) -> N;
-get_face_num([{group,_,N}|_]) -> N.
+get_face_num([{group,_,N}|_]) -> N;
+get_face_num([{name,_,N}|_]) -> N.
 
 skip_upto_name([{eof,N}]=T) -> {T,N};
 skip_upto_name([{name,_,N}|_]=T) -> {T,N};
 skip_upto_name([_|T]) -> skip_upto_name(T).
 
+make_objects([{_Name,0}|T], Fs, Template) ->
+    %% Ignore object with 0 faces.
+    make_objects(T, Fs, Template);
 make_objects([{Name,N}|T], Fs0, Template) ->
     {Ftab,Fs} = split(Fs0, N, []),
     Mesh0 = e3d_mesh:clean_faces(Template#e3d_mesh{fs=Ftab}),
@@ -176,6 +233,18 @@ parse(["g"|Names], Ost) ->
 parse(["o"], Ost) -> Ost;
 parse(["o",Name|_], Ost) ->
     remember_name(Name, Ost);
+parse(["s","off"], Ost) ->
+    remember_sgroup(0, Ost);
+parse(["s",Sg0|T], Ost) ->
+    try
+	Sg = list_to_integer(Sg0),
+	T = [],
+	remember_sgroup(Sg, Ost)
+    catch
+	error:_ ->
+	    %% Ignore bad "s" statement.
+	    Ost
+    end;
 parse(["usemtl"|[Mat|_]], Ost) ->
     Ost#ost{mat=[list_to_atom(Mat)]};
 parse(["mtllib",FileName], #ost{dir=Dir}=Ost) ->
@@ -201,6 +270,9 @@ remember_group(Names, #ost{f=Ftab,g=Gs}=Ost) ->
 
 remember_name(Name, #ost{f=Ftab,g=Gs}=Ost) ->
     Ost#ost{g=[{name,Name,length(Ftab)}|Gs]}.
+
+remember_sgroup(Sg, #ost{s=Sgs,f=Ftab}=Ost) ->
+    Ost#ost{s=[{Sg,length(Ftab)}|Sgs]}.
     
 collect_vs([V|Vs], Ost) ->
     [collect_vtxref(V, Ost)|collect_vs(Vs, Ost)];
