@@ -3,13 +3,12 @@
 %%
 %%     The UV parametrisation algorithms.
 %%
-%%  Copyright (c) 2002-2007 Dan Gudmundsson, Raimo Niskanen,
-%%                     Bjorn Gustavsson
+%%  Copyright (c) 2002-2009 Dan Gudmundsson, Raimo Niskanen, Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: auv_mapping.erl 210 2008-01-05 17:43:04Z bjorng $
+%%     $Id$
 %%
 
 %%%%%% Least Square Conformal Maps %%%%%%%%%%%%
@@ -44,10 +43,10 @@
 
 %% Internal exports. 
 -export([model_l2/5]).
-
 -export([lsq/2, lsq/3,  % Debug entry points
 	 find_pinned/2,
-	 find_pinned_from_edges/2]). 
+	 find_pinned_from_edges/2,
+	 split_edges_1/2]).
 
 -include("wings.hrl").
 -include("auv.hrl").
@@ -93,7 +92,7 @@ map_chart_2(Op,C, Loop, Pinned, We) ->  volproject(Op,C, Pinned, Loop,We).
 
 volproject(Type,Chart,_Pinned,{_,BEdges},We) ->
     {Center,Axes,LoopInfo} = find_axes(Chart,BEdges,We),
-%%    io:format("Res: ~p ~n",[{Center,Axes}]),
+    %%io:format("Res: ~p ~n",[{Center,Axes}]),
     Rot = rot_mat(Axes),
     CalcUV = case Type of 
 		 cyl -> fun cyl/1; 
@@ -107,13 +106,13 @@ volproject(Type,Chart,_Pinned,{_,BEdges},We) ->
 		end,
     Vs1 = lists:sort([{V,Transform(V)} || V <- Vs0]),
     Tagged = leftOrRight(LoopInfo, Chart, We#we{vp=gb_trees:from_orddict(Vs1)}),
+    %%io:format("Tagged ~w~n",[gb_sets:to_list(Tagged)]),
     [{V,fix_positions(V,Pos,CalcUV(Pos),Tagged)} || {V,Pos} <- Vs1].
 
 sphere({X,Y,Z}) ->    
-     S = catchy(catch math:atan2(X,Z)/math:pi()),	
-     T = math:acos(clamp(-Y))/math:pi()-0.5,
-     {S,T,0.0}.
-%%     {X,Y,0.0}.
+    S = catchy(catch math:atan2(X,Z)/math:pi()),	
+    T = math:acos(clamp(-Y))/math:pi()-0.5,
+    {S,T,0.0}.
 
 cyl({X,Y,Z}) ->
     S = catchy(catch math:atan2(X,Z)/math:pi()),
@@ -195,21 +194,13 @@ find_axes(Fs,BEdges,We) ->
 
 forms_closed_object(BEdges0,ChartNormal,We=#we{name=#ch{emap=Emap}}) ->
     BEdges = [{auv_segment:map_edge(Edge,Emap),BE} || BE = #be{edge=Edge} <- BEdges0],
-    case is_an_8(BEdges) of
+    case is_an_8(BEdges, false) of
 	false -> undefined;
 	Edge -> 
-	    {L1,L2,Link,LinkR} = split_edges(Edge,BEdges),
-	    North = case L1 of 
-			[] -> wings_vertex:pos((hd(Link))#be.vs,We);
-			_ -> center(L1,We)
-		    end,
-	    South = case L2 of 
-			[] -> wings_vertex:pos((lists:last(Link))#be.ve,We);
-			_ -> center(L2,We)
-		    end,
+	    {North,South,Link,LinkR} = split_edges(Edge,BEdges,We),
 	    NorthSouth = e3d_vec:sub(North,South),
 	    Center = e3d_vec:average(North,South),
-%%	    io:format("Temp: ~p ~n",[{North,South,Center}]),
+	    %%io:format("Temp: ~p ~n",[{North,South,Center}]),
 	    LC = center(Link,We),
 	    LCdir0 = e3d_vec:sub(LC,Center),
 	    LCdir = case e3d_vec:len(LCdir0) > 0.0005 of
@@ -230,11 +221,21 @@ calc_axis(Y0,Z0) ->
     Z = e3d_vec:norm(e3d_vec:cross(X,Y)),
     {X,Y,Z}.
 
-is_an_8([]) -> false;
-is_an_8([{E,_}|R]) -> %% Hmm we must take them in order
+is_an_8([],E) ->
+    E;
+is_an_8([{E,_},{E,_}|R],_) -> %% Skip these
+    is_an_8(R, E);
+is_an_8([{E,_}|R],HaveRemoved) -> %% Hmm we must take them in order
     case lists:keysearch(E,1,R) of %% O(N2) I know..
-	false -> is_an_8(R);
-	_ -> E
+	false -> is_an_8(R,HaveRemoved);
+	true when HaveRemoved =/= false ->
+	    E;
+	_ ->
+	    case reverse(R) of
+		[{E,_}|R2] ->
+		    is_an_8(reverse(R2), E);
+		_ -> E
+	    end
     end.
 
 %%  Split edges splits into three parts two loops 
@@ -245,58 +246,68 @@ is_an_8([{E,_}|R]) -> %% Hmm we must take them in order
 %%   \_/--\_|     => 2 loops: mnoabc fghijk
 %%   onmdekji     =>    link: def
 %% 
-%d(L) -> %% DBG BUGBUG remove
-%    lists:map(fun({E,_BE}) -> E end,L).
-    
-split_edges(Edge,Bes) ->
-    {Loop1,Loop2,Link} = split_edges_1(Edge,Bes),
-    Get = fun(L) -> lists:map(fun({_,BE}) -> BE end,L) end,
-    LinkR = (((Bes -- Loop1) -- Loop2) -- Link),
-    {Get(Loop1),Get(Loop2),Get(Link),Get(LinkR)}.
+%% d(L) -> %% DBG
+%%     lists:map(fun({E,_BE}) -> E end,L).
+
+getEs(L) ->
+    lists:map(fun({_E,BE}) -> BE end,L).
+
+split_edges(Edge,Bes,We) ->
+    {L1,L2,Link} = split_edges_1(Edge,Bes),
+    %% Reorder so that the pinned vertices are longest from each other
+    North = case L1 of
+		[] -> wings_vertex:pos((hd(Link))#be.vs,We);
+		_  -> center(L1,We)
+	    end,
+    South = case L2 of
+		[] -> wings_vertex:pos((lists:last(Link))#be.ve,We);
+		_  -> center(L2,We)
+	    end,
+    LinkR = (((getEs(Bes) -- L1) -- L2) -- Link),
+    {North,South,Link,LinkR}.
 
 split_edges_1(Edge,Bes) ->
-%    io:format("Split: ~w ~w~n",[Edge,d(Bes)]),
-    {P1r,Pr1} = find_split(Edge,Bes,[]),
-    {P2r,Pr2} = find_split(Edge,Pr1,[]),
-    {Loop1,Link1} = find_link(Pr1,tl(P2r),[hd(P2r)]),
-    {Loop2,Link2} = find_link(tl(P1r),Pr2++reverse(tl(P1r)),reverse(Link1)),
-%    io:format("L1:~w~nLink1:~w~n",[d(Loop1),d(Link1)]),
-%    io:format("L2:~w~nLink2:~w~n",[d(Loop2),d(Link2)]),
-    {Loop1,Loop2,Link2}.
+%%     io:format("Split: ~w ~w~n",[Edge,d(Bes)]),
+    {Before,BE1,After} = find_split(Edge,Bes,[]),
+    {LeftLoop0,BE2,RightLoop0} = find_split(Edge,After,[BE1]),
+    LeftLoop  = LeftLoop0 ++ [BE2],
+    %% NOTE: Order is important below
+    RightLoop = reverse(RightLoop0 ++ Before),
+    {Loop1,Link1} = find_link(LeftLoop,  reverse(LeftLoop), []),
+    {Loop2,Link2} = find_link(RightLoop, reverse(RightLoop), []),
+%%     io:format("L1:~w~nL2:~w~nLink1:~w~nLink2:~w~n~n",
+%% 	      [Loop1,(Loop2),(Link1),(Link2)]),
+    Link = reorder_link(Link2++reverse(Link1)),
+%%     io:format("Link:~w~n",[d(Link)]),
+    {getEs(Loop1),getEs(Loop2),getEs(Link)}.
 
-find_split(Edge,[G={Edge,_Be}|Bes],Rest) -> {[G|Rest],Bes};
-find_split(Edge,[This|Bes],Rest) ->
-    find_split(Edge,Bes,[This|Rest]).
-    
-find_link([{E,_}|C1],[G={E,_}|C2],Acc) ->
-    find_link(C1,remove_last(C2,E),[G|Acc]);
-find_link(_C1,C2,Acc) -> 
-    case is_an_8(C2) of 
-	false -> {C2,Acc};
-	Edge ->
-	    [{SE,_Be}|_] = C2,
-	    {L1,L2,Li} = split_edges_1(Edge,reverse(C2)),
-	    RealLoop = 
-		case lists:keysearch(SE,1,L1) of
-		    false -> L1;
-		    _ -> L2
-		end,
-	    Res = get_until(C2,Li,[]),
-	    {RealLoop,Res ++ Acc}
-    end.
+find_split(Edge,[G={Edge,_}|Bes],Acc) -> {reverse(Acc),G,Bes};
+find_split(Edge,[This|Bes],Acc) ->
+    find_split(Edge,Bes,[This|Acc]).
 
-get_until(_, [], Acc) -> Acc;     
-get_until([L={E,_}|R1],[{E,_}|R2],Acc) ->
-    get_until(R1,R2,[L|Acc]);
-get_until([L|R1],R2,Acc) ->
-    get_until(R1,R2,[L|Acc]).
+find_link([{E,_}|_],[{E,_}|_],Link = [{E,_}|_]) ->
+    {[],Link};
+find_link([G={E,_}|C1],[{E,_}|C2],Link) ->
+    find_link(C1,C2,[G|Link]);
+find_link(C1,_,Link) ->
+    find_loop(C1,Link,[]).
 
-remove_last([],_) -> [];
-remove_last(C,Edge) ->
-    case reverse(C) of
-	[{Edge,_}|R] -> reverse(R);
-	_ -> C
-    end.
+find_loop([{E,_}|_],[{E,_}|_]=Link, Loop) ->
+    {Loop,Link};
+find_loop([G|C1],Link,Loop) ->
+    find_loop(C1,Link,[G|Loop]);
+find_loop([],[],Loop) -> {Loop,[]};
+find_loop([],Link,[]) -> {[],Link}.
+
+reorder_link([]) -> [];
+reorder_link(A=[_]) -> A;
+reorder_link(Ok  = [{_,#be{ve=V}},{_,#be{vs=V}}|_]) -> Ok;
+reorder_link(Rev = [{_,#be{vs=V}},{_,#be{ve=V}}|_]) ->
+    %% reverse(Rev);  %% Correctness asserted below
+    reorder_link(reverse(Rev));
+reorder_link(Other) ->
+    io:format("Other: ~w~n",[Other]),
+    exit(internal_error).
 
 %%%% Uncomplete fixme.. BUGBUG
 %%% I can't get this to work satisfactory..aarg.
@@ -426,20 +437,15 @@ lsqcm(Fs, none, Loop, We) ->
 lsqcm(Fs, Pinned, _Loop, We) ->
     ?DBG("Project and tri ~n", []),
     LSQState = lsq_setup(Fs,We,Pinned),
-    case lsq(LSQState, Pinned) of
-	{error, What} ->
-	    ?DBG("TXMAP error ~p~n", [What]),
-	    exit({txmap_error, What});
-	{ok,Vs2} ->
-	    %%?DBG("LSQ res ~p~n", [Vs2]),
-	    Patch = fun({Idt, {Ut,Vt}}) -> {Idt,{Ut,Vt,0.0}} end,
-	    Vs3 = lists:sort(lists:map(Patch, Vs2)),
-	    TempVs = gb_trees:from_orddict(Vs3),
-	    Area = fs_area(Fs, We, 0.0),
-	    MappedArea = fs_area(Fs, We#we{vp=TempVs}, 0.0),
-	    Scale = Area/MappedArea,
-	    scaleVs(Vs3,math:sqrt(Scale),[])
-    end.
+    {ok,Vs2} = lsq(LSQState, Pinned),
+    %%?DBG("LSQ res ~p~n", [Vs2]),
+    Patch = fun({Idt, {Ut,Vt}}) -> {Idt,{Ut,Vt,0.0}} end,
+    Vs3 = lists:sort(lists:map(Patch, Vs2)),
+    TempVs = gb_trees:from_orddict(Vs3),
+    Area = fs_area(Fs, We, 0.0),
+    MappedArea = fs_area(Fs, We#we{vp=TempVs}, 0.0),
+    Scale = Area/MappedArea,
+    scaleVs(Vs3,math:sqrt(Scale),[]).
 
 scaleVs([{Id, {X,Y,_}}|Rest],Scale,Acc) 
   when is_float(X), is_float(Y), is_float(Scale) ->
