@@ -3,16 +3,16 @@
 %%
 %%     Render and capture a texture.
 %%
-%%  Copyright (c) 2002-2006 Dan Gudmundsson, Bjorn Gustavsson
+%%  Copyright (c) 2002-2008 Dan Gudmundsson, Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: auv_texture.erl 177 2007-12-21 18:56:58Z bjorng $
+%%     $Id$
 %%
 
 -module(auv_texture).
--export([get_texture/1, get_texture/2, draw_options/0]).
+-export([get_texture/2, draw_options/0]).
 
 -compile(export_all).
 
@@ -317,10 +317,6 @@ drop_flags(Id) ->
 %% Texture creation
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-get_texture(St) ->    
-    Ops = list_to_prefs(get_pref(tx_prefs, list_to_prefs(#opt{}))),
-    get_pref(St, Ops).
-
 get_texture(St = #st{bb=#uvstate{}}, {Options,Shaders}) ->
     Compiled = compile_shaders(Options#opt.renderers,Shaders),
     Passes = get_passes(Options#opt.renderers,{Shaders,Compiled}),
@@ -501,7 +497,7 @@ get_texture(Wc, Wd, Hc, Hd, {W,H}=Info, DL, UsingFbo, ImageAcc)
   when Wc < Wd, Hc < Hd ->
     gl:pixelStorei(?GL_UNPACK_ALIGNMENT, 1),
     gl:clearColor(1, 1, 1, 1),
-%%    gl:shadeModel(?GL_SMOOTH),
+    gl:shadeModel(?GL_SMOOTH),
     gl:disable(?GL_CULL_FACE),
     gl:disable(?GL_LIGHTING),
     texture_view(Wc, Wd, Hc, Hd),
@@ -915,14 +911,16 @@ vs_lines([A|R=[B|_]],Last) ->
 vs_lines([B],Last) ->
     [B,Last].
 
-merge_mats([This={MatName,_Def}|R], Mats) ->
-    case gb_trees:is_defined(MatName,Mats) of
+merge_mats([{MatName,Mat}|R], Mats) ->
+    case gb_trees:is_defined(MatName, Mats) of
 	true ->
-	    merge_mats(R,Mats);
+	    merge_mats(R, Mats);
 	false ->
-	    merge_mats(R,gb_trees:add(This, Mats))
+	    %% This can happen if the material has been deleted
+	    %% from the Outliner after entering the AutoUV window.
+	    merge_mats(R, gb_trees:insert(MatName, Mat, Mats))
     end;
-merge_mats([],Mats) -> Mats.
+merge_mats([], Mats) -> Mats.
 	     
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Builtin Shader Passes
@@ -1276,6 +1274,8 @@ render_lights([{Lx,Ly,Lz}| Ligths], PM, Dist, C={Cx,Cy,Cz}, LP1,Ts = #ts{charts=
     
     R = fun(#fs{vs=Vs,id=Id}) ->
 		%% gl:color4ubv(id(Id)),
+		%% XXX The following line is broken. id/1 returns a tuple,
+		%% but a list of tuples is needed for argument 3.
 		gl:uniform4fv(IdPos, 1, id(Id)),
 		gl:drawElements(?GL_TRIANGLES,length(Vs),
 				?GL_UNSIGNED_INT,Vs)
