@@ -3,12 +3,12 @@
 %%
 %%     Connect/Cut mode plugin.
 %%
-%%  Copyright (c) 2004 Dan Gudmundsson
+%%  Copyright (c) 2004-2008 Dan Gudmundsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_connect_tool.erl 177 2007-12-21 18:56:58Z bjorng $
+%%     $Id$
 %%
 -module(wpc_connect_tool).
 
@@ -20,8 +20,6 @@
 
 -include_lib("wings.hrl").
 
--import(lists, [foldl/3,last/1,member/2,reverse/1,reverse/2,
-		seq/2,sort/1]).
 %% State info
 -record(cs, {v=[],  %% Connected vertices
 	     last,  %% Last vertex selected (for ending loop)
@@ -188,6 +186,16 @@ handle_connect_event1({action,Action}, #cs{st=St0}=C) ->
 	    St = fake_selection(St0),
 	    wings_view:command(aim, St),
 	    update_connect_handler(C);
+	{view,highlight_aim} ->
+	    St = fake_selection(St0),
+	    case wings_pref:get_value(highlight_aim_at_unselected) of
+	      false ->
+	          wings_view:command(aim, St0),
+	          update_connect_handler(C);
+	      true ->
+	          wings_view:command(aim, St),
+	          update_connect_handler(C)
+	    end;
 	{view,Cmd} ->
 	    case wings_view:command(Cmd, St0) of
 		keep ->
