@@ -3,14 +3,14 @@
 %%
 %%     Plug-in for extending an edge to the intersection with a plane.
 %%
-%%  Copyright (c) 2004 Bjorn Gustavsson.
+%%  Copyright (c) 2004-2009 Bjorn Gustavsson.
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
 %%  Contributed by elrond79.
 %%
-%%     $Id: wpc_intersect_edge.erl 177 2007-12-21 18:56:58Z bjorng $
+%%     $Id$
 %%
 %%  2000-10-09:  Fixed undo bug (had forgotten to use "{save_state, ...}")
 %%  2000-10-01:  Incorporated help text suggestions from Puzzled Paul
@@ -25,15 +25,25 @@
 -export([init/0,menu/2,command/2]).
 
 -include_lib("wpc_intersect.hrl").
--import(lists, [splitwith/2,member/2]).
+-import(lists, [member/2]).
 
 init() ->
     true.
 
+menu({edge},Menu) ->
+    lists:reverse(parse(Menu, [], false));
+menu(_,Menu) ->
+    Menu.
+
+parse([], NewMenu, true) ->
+    NewMenu;
+parse([], NewMenu, false) ->
+    [menu_item()|NewMenu];
+parse([A = {_,{extrude,_}}|Rest], NewMenu, false) ->
+    parse(Rest, [menu_item(),A|NewMenu], true);
+parse([Elem|Rest], NewMenu, Found) ->
+    parse(Rest, [Elem|NewMenu], Found).
 
-menu({edge}, Menu) ->
-    Menu ++ [separator,menu_item()];
-menu(_,Menu) -> Menu.
 
 command({edge,{intersect,Plane}},St) ->
     intersect(Plane, St);
@@ -61,9 +71,9 @@ standard_planes_menu() ->
     [standard_plane_fun(x),
      standard_plane_fun(y),
      standard_plane_fun(z),
-     {advanced,separator},
-     {advanced,standard_plane_fun(last_axis)},
-     {advanced,standard_plane_fun(default_axis)}].
+     separator,
+     standard_plane_fun(last_axis),
+     standard_plane_fun(default_axis)].
 
 standard_plane_fun(Vec) ->
     F = fun(1, Ns) ->
@@ -130,8 +140,6 @@ validateSel(Edges, We, valid) ->
 validateSel(_Edges, _We, invalid) ->
     invalid.
     
-validateSelAcc(_SelectedEdges,_EdgeTab,invalid) ->
-    invalid;
 validateSelAcc([],_,_VertexAcc)->
     valid;
 validateSelAcc([SelEdge|OtherSelEdges], EdgeTab, VertexAcc) ->
