@@ -3,14 +3,14 @@
 %%
 %%     Plug-in for moving vertice(s) to the intersection of a line and plane
 %%
-%%  Copyright (c) 2004 Bjorn Gustavsson.
+%%  Copyright (c) 2004-2009 Bjorn Gustavsson.
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
 %%  Contributed by elrond79.
 %%
-%%     $Id: wpc_intersect_vertex.erl 245 2008-02-18 06:30:09Z optigon $
+%%     $Id$
 %%
 %%  2000-10-01:  Changed help text to incorporate suggestions by Puzzled Paul
 %%  2000-09-21:  Normalized LD and PN (so dot product not <.001 if either is very
@@ -59,17 +59,7 @@ command(_,_) -> next.
 %% creates the menu item for the vertex intersect command.
 
 menu_item() ->
-    [{basic, {?__(1,"Intersect"),{intersect,[{?__(2,"Stay on line"), {'VALUE', submenu_items(1)},
-				       ?__(3,"Move vertex along LINE of chosen direction to intersection "
-				       "with chosen PLANE")},
-				      {?__(4,"Stay on plane"), {'VALUE', submenu_items(2)},
-				       ?__(5,"Move vertex along PLANE with chosen normal to intersection "
-				       "with chosen LINE")},
-				      {?__(6,"Pick all"), {'VALUE', submenu_items(3)},
-				       ?__(7,"Move vertex to intersection of LINE of chosen direction and location "
-				       "with PLANE with chosen normal and location")}]}}},
-     {advanced, {?__(8,"Intersect"),{intersect,fun adv_submenu/2}}}].
-
+    [{?__(8,"Intersect"),{intersect,fun adv_submenu/2}}].
 
 submenu_items(1) ->
     {stay_on_line,
