@@ -9,7 +9,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_opengl.erl 211 2008-01-06 08:06:06Z bjorng $
+%%     $Id$
 
 -module(wpc_opengl).
 
@@ -21,17 +21,11 @@
 
 -export([init/0,menu/2,command/2]).
 
--import(lists, [foldl/3,map/2,foreach/2,reverse/1,seq/2,
-		flat_length/1,append/1,append/2]).
+-import(lists, [foldl/3,map/2,foreach/2,reverse/1]).
 
 -export([calcTS/4]). %% debug
 -export([expand_arealights/1]). % debug
 
-%% UNTIL ESDL catches up..
--ifndef(GL_DEPTH_CLAMP_NV).
--define(GL_DEPTH_CLAMP_NV,	16#864F).
--endif.
-
 -define(STENCIL_INIT, 128).
 
 -record(r,
@@ -228,7 +222,7 @@ prepare_mesh(We0=#we{light=none},SubDiv,RenderAlpha,RenderBumps,Wes,Shapes,St) -
     wings_pb:update(Step+Start, ?__(3,"Generating mesh data")),
     FN0	     = [{Face,wings_face:normal(Face, We)} || Face <- gb_trees:keys(We#we.fs)],
     FVN	     = wings_we:normals(FN0, We),  %% gb_tree of {Face, [VInfo|Normal]}
-    MatFs0   = wings_facemat:mat_faces(FVN, We), %% sorted by mat..
+    MatFs0   = wings_facemat:mat_faces(FVN, We), %% sorted by mat...
     MatFs    = patch_tangent_space(MatFs0, We, []),
     PAble = programmable(),
     wings_pb:pause(),
@@ -258,7 +252,7 @@ create_dls(St0, Attr, Shadows, Bumps) ->
 				    Wes, length(Objects), St)
 	       end, [], Objects),
     Ls = if
-	     Shadows or Bumps -> %% Needs per-light data..
+	     Shadows or Bumps -> %% Needs per-light data...
 		 Ls0  = expand_arealights(wpa:lights(St)),
 		 Mats = St#st.mat,
 		 wings_pb:update(0.95, ?__(1,"Generating shadows and bump data per light ")),
@@ -773,33 +767,11 @@ accPerspective(Fovy, Aspect, ZNear, ZFar,
 
 jitter(1) ->
     [{0.0,0.0}];
-jitter(2) ->
-    [{0.25,0.75},{0.75,0.25}];
-jitter(3) ->
-    [{0.5033922635,0.8317967229},
-     {0.7806016275,0.2504380877},
-     {0.22261828938,0.4131553612}];
 jitter(4) ->
     [{0.375,0.25},{0.125,0.75},{0.875,0.25},{0.625,0.75}];
-jitter(5) ->
-    [{0.375,0.25},{0.125,0.75},{0.875,0.25},{0.625,0.75}];
-jitter(6) ->
-    [{0.4646464646,0.4646464646},{0.1313131313,0.7979797979},
-     {0.5353535353,0.8686868686},{0.8686868686,0.5353535353},
-     {0.7979797979,0.1313131313},{0.2020202020,0.2020202020}];
 jitter(8) ->
     [{0.5625,0.4375},{0.0625,0.9375},{0.3125,0.6875},{0.6875,0.8125},
      {0.8125,0.1875},{0.9375,0.5625},{0.4375,0.0625},{0.1875,0.3125}];
-jitter(9) ->
-    [{0.5,0.5},{0.1666666666,0.9444444444},{0.5,0.1666666666},
-     {0.5,0.8333333333},{0.1666666666,0.2777777777},
-     {0.8333333333,0.3888888888},{0.1666666666,0.6111111111},
-     {0.8333333333,0.7222222222},{0.8333333333,0.0555555555}];
-jitter(12) ->
-    [{0.4166666666,0.625},{0.9166666666,0.875},{0.25,0.375},
-     {0.4166666666,0.125},{0.75,0.125},{0.0833333333,0.125},{0.75,0.625},
-     {0.25,0.875},{0.5833333333,0.375},{0.9166666666,0.375},
-     {0.0833333333,0.625},{0.583333333,0.875}];
 jitter(16) ->
     [{0.375,0.4375},{0.625,0.0625},{0.875,0.1875},{0.125,0.0625},
      {0.375,0.6875},{0.875,0.4375},{0.625,0.5625},{0.375,0.9375},
@@ -1175,7 +1147,7 @@ draw_vtx_color([{Face,[[Col1|N1],[Col2|N2],[Col3|N3]|_BUG]}|Faces], We) ->
     draw_vtx_color(Faces,We).
 
 do_draw_faces([],_,_) -> ok;
-%%% Unprogrammable cards..
+%%% Unprogrammable cards...
 do_draw_faces([{Face, [[UV1|TBN1],[UV2|TBN2],[UV3|TBN3]|_BUG]}|Fs],Tex, We) ->
     [V1,V2,V3|_] = wings_face:vertex_positions(Face, We),
     gl:normal3fv(element(3, TBN1)),
@@ -1248,6 +1220,7 @@ texCoord(UV = {_,_}, true) ->
 texCoord(_, true) ->
     gl:texCoord2fv({0.0,0.0});
 texCoord(_, false) -> ok.
+
 texCoord(UV = {_,_}, true, Unit) ->
     gl:multiTexCoord2fv(Unit, UV);
 texCoord(_, true, Unit) ->
