@@ -3,20 +3,18 @@
 %%
 %%     Renderer implemented using the Shading Language in OpenGL.
 %%
-%%  Copyright (c) 2004 Bjorn Gustavsson
+%%  Copyright (c) 2004-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_opengl_sl.erl 176 2007-12-21 18:44:56Z bjorng $
+%%     $Id$
 
 -module(wpc_opengl_sl).
 
 -export([init/0,menu/2,command/2]).
 -export([enable/0,disable/0]).
 
--import(lists, [reverse/1,reverse/2]).
-
 enable() -> wpa:pref_set(?MODULE, enabled, true).
 
 disable() -> wpa:pref_delete(?MODULE, enabled).
@@ -41,7 +39,7 @@ command({file,{render,{opengl_sl,Ask}}}, St) ->
     render(Ask, St);
 command(_, _) -> next.
 
-render(Ask, St) ->
+render(Ask, _St) ->
     io:format("~p\n", [Ask]),
     keep.
 
