@@ -8,12 +8,12 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_ncube.erl 315 2008-04-13 20:44:00Z optigon $
+%%     $Id$
 %%
 
 -module(wpc_ncube).
 -export([init/0,menu/2,command/2]).
--import(math, [sqrt/1,cos/1,sin/1,pi/0,pow/2,exp/1]).
+-import(math, [cos/1,sin/1,pi/0]).
 -include("e3d.hrl").
 -include("wings.hrl").
 
