@@ -3,12 +3,12 @@
 %%
 %%     Console for Wings.
 %%
-%%  Copyright (c) 2004-2005 Raimo Niskanen
+%%  Copyright (c) 2004-2009 Raimo Niskanen
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_console.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_console).
@@ -28,10 +28,8 @@
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
--import(lists, [reverse/1,reverse/2,foreach/2,foldl/3,
-		nthtail/2,sublist/2,sublist/3,
-		flatten/1,flatten/2]).
--import(wings_util, [min/2,max/2]).
+-import(lists, [reverse/1,reverse/2,foldl/3]).
+-import(erlang, [min/2,max/2]).
 
 %% Debug exports
 -export([code_change/0,get_state/0]).
@@ -433,6 +431,15 @@ forward({io_request,From,ReplyAs,Req0}) ->
     Req = forward_1(Req0),
     group_leader() ! {io_request,From,ReplyAs,Req}.
     
+forward_1({put_chars,unicode,Chars}) when is_binary(Chars) ->
+    {put_chars,Chars};
+forward_1({put_chars,unicode,Chars}) when is_list(Chars) ->
+    try
+	{put_chars,list_to_binary(Chars)}
+    catch
+	error:badarg ->
+	    {put_chars,filter_chars(Chars)}
+    end;
 forward_1({put_chars,Chars}) when is_list(Chars) ->
     try
 	{put_chars,list_to_binary(Chars)}
@@ -442,6 +449,8 @@ forward_1({put_chars,Chars}) when is_list(Chars) ->
     end;
 forward_1({put_chars,Chars}=Req) when is_binary(Chars) ->
     Req;
+forward_1({put_chars,unicode,Mod,Func,Args}) ->
+    forward_1({put_chars,unicode,apply(Mod, Func, Args)});
 forward_1({put_chars,Mod,Func,Args}) ->
     forward_1({put_chars,apply(Mod, Func, Args)}).
 
@@ -459,6 +468,15 @@ filter_chars([]) -> [].
 
 io_request(State, {put_chars,Chars}) ->
     {put_chars(State, Chars),ok,forward};
+io_request(State, {put_chars,unicode,Chars}) ->
+    {put_chars(State, unicode:characters_to_list(Chars)),ok,forward};
+io_request(State, {put_chars,unicode,Mod,Func,Args}) ->
+    case catch apply(Mod, Func, Args) of
+	Chars when is_list(Chars); is_binary(Chars) ->
+	    io_request(State, {put_chars,unicode,Chars});
+	_ ->
+	    {State,{error,Func},error}
+    end;
 io_request(State, {put_chars,Mod,Func,Args}) ->
     case catch apply(Mod, Func, Args) of
 	Chars when is_list(Chars); is_binary(Chars) ->
@@ -471,7 +489,16 @@ io_request(State, {requests,Requests}) when is_list(Requests) ->
 io_request(State, {setopts,Opts}) when is_list(Opts) ->
     {State,{error,badarg},error};
 io_request(State, Request) ->
-    {State,{error,{request,Request}}}.
+    %% Probably a new version of Erlang/OTP with extensions to the
+    %% I/O protocol. We could generate an error here, but the
+    %% stack dump in wings_crash.dump would generate the caller of
+    %% io:format/2 in the main Wings process with no indication that
+    %% this module is the culprit. Therefore, we choose to ignore
+    %% the request, but write a message to the console to point out
+    %% the problem.
+    S = io_lib:format("Internal error in Console - unknown I/O request:\n~P\n",
+		      [Request,10]),
+    {put_chars(State, iolist_to_binary(S)),ok,forward}.
 
 io_request_loop([], Result) ->
     Result;
