@@ -3,12 +3,12 @@
 %%
 %%     This module draws objects using OpenGL.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_draw.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_draw).
@@ -19,13 +19,16 @@
 	 split/3,original_we/1,update_dynamic/2,join/1,abort_split/1,
 	 face_ns_data/1]).
 
+% export for plugins that need to draw stuff
+-export([pump_vertices/1]).
+
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 -include("e3d.hrl").
 
--import(lists, [foreach/2,last/1,reverse/1,reverse/2,member/2,
-		foldl/3,merge/1,sort/1,keysort/2,any/2,seq/2]).
+-import(lists, [foreach/2,reverse/1,reverse/2,member/2,
+		foldl/3,sort/1,keysort/2]).
 
 -record(split,
 	{static_vs,
@@ -77,12 +80,15 @@ prepare_fun(#dlo{}=D, [#we{perm=Perm}|Wes]) when ?IS_NOT_VISIBLE(Perm) ->
     prepare_fun(D, Wes);
 prepare_fun(#dlo{src_we=We,split=#split{}=Split}=D, [We|Wes]) ->
     {D#dlo{src_we=We,split=Split#split{orig_we=We}},Wes};
+
 prepare_fun(#dlo{src_we=We}=D, [We|Wes]) ->
     %% No real change - take the latest We for possible speed-up
     %% of further comparisons.
     {D#dlo{src_we=We},Wes};
+
 prepare_fun(#dlo{src_we=#we{id=Id}}=D, [#we{id=Id}=We1|Wes]) ->
     prepare_fun_1(D, We1, Wes);
+
 prepare_fun(#dlo{}, Wes) ->
     {deleted,Wes}.
 
@@ -116,10 +122,9 @@ invalidate_by_mat(#dlo{work=none,vs=none,smooth=none,proxy_faces=none}=D, _) ->
     D;
 invalidate_by_mat(#dlo{src_we=We}=D, Changed) ->
     Used = wings_facemat:used_materials(We),
-    case ordsets:intersection(Used, Changed) of
-	[] -> D;
-	[_|_] ->
-	    D#dlo{work=none,edges=none,vs=none,smooth=none,proxy_faces=none}
+    case ordsets:is_disjoint(Used, Changed) of
+	true -> D;
+	false -> D#dlo{work=none,edges=none,vs=none,smooth=none,proxy_faces=none}
     end.
 
 invalidate_sel(#dlo{src_we=#we{id=Id},src_sel=SrcSel}=D,
@@ -243,16 +248,17 @@ update_needed_2(CommonNeed, St) ->
 update_needed_fun(#dlo{src_we=#we{perm=Perm}=We}=D, _, _, _)
   when ?IS_LIGHT(We), ?IS_VISIBLE(Perm) ->
     D#dlo{needed=[light]};
-update_needed_fun(#dlo{src_we=#we{id=Id,he=Htab},proxy_data=Pd}=D,
+update_needed_fun(#dlo{src_we=#we{id=Id,he=Htab,pst=Pst},proxy_data=Pd}=D,
 		   Need0, Wins, _) ->
     Need1 = case gb_sets:is_empty(Htab) orelse
 		not wings_pref:get_value(show_edges) of
 		false -> [hard_edges|Need0];
 		true -> Need0
 	    end,
+	Need2 = wings_plugin:check_plugins(update_dlist,Pst) ++ Need1,
     Need = if
-	       Pd =:= none -> Need1;
-	       true -> [proxy|Need1]
+	       Pd =:= none -> Need2;
+	       true -> [proxy|Need2]
 	   end,
     D#dlo{needed=more_need(Wins, Id, Need)}.
 
@@ -282,6 +288,7 @@ wins_of_same_class() ->
 	_ -> [wings_wm:this()]
     end.
 
+
 %%
 %% Rebuild all missing display lists, based on what is
 %% actually needed in D#dlo.needed.
@@ -318,11 +325,9 @@ update_fun_2(smooth, #dlo{smooth=none}=D, St) ->
     We = wings_proxy:smooth_we(D),
     {List,Tr} = smooth_dlist(We, St),
     D#dlo{smooth=List,transparent=Tr};
-update_fun_2({vertex,PtSize}, #dlo{vs=none,src_we=We}=D, _) ->
+update_fun_2({vertex,_PtSize}, #dlo{vs=none,src_we=We}=D, _) ->
     UnselDlist = gl:genLists(1),
     gl:newList(UnselDlist, ?GL_COMPILE),
-    gl:pointSize(PtSize),
-    gl:color3b(0, 0, 0),
     gl:'begin'(?GL_POINTS),
     pump_vertices(visible_vertices(We)),
     gl:'end'(),
@@ -348,6 +353,17 @@ update_fun_2(normals, D, _) ->
     make_normals_dlist(D);
 update_fun_2(proxy, D, St) ->
     wings_proxy:update(D, St);
+
+%%%% Check if plugins using the Pst need a draw list updated.
+update_fun_2({plugin,{Plugin,{_,_}=Data}},#dlo{plugins=Pdl}=D,St) ->
+    case lists:keytake(Plugin,1,Pdl) of
+	    false ->
+		  Plugin:update_dlist(Data,D,St);
+	    {_,{Plugin,{_,none}},Pdl0} ->
+	      Plugin:update_dlist(Data,D#dlo{plugins=Pdl0},St);
+		_ -> D
+	end;
+
 update_fun_2(_, D, _) -> D.
 
 make_edge_dl(Ns) ->
@@ -413,7 +429,8 @@ visible_vertices(#we{vp=Vtab0}=We) ->
 				      [{vertex,position}]),
 	    sofs:to_external(sofs:image(Vtab, Vis))
     end.
-    
+
+
 %%%
 %%% Update the selection display list.
 %%%
@@ -621,7 +638,7 @@ insert_vtx_data([], _, Acc) -> reverse(Acc).
 split_vs_dlist(Vs, StaticVs, {vertex,SelVs0}, #we{vp=Vtab}=We) ->
     case wings_pref:get_value(vertex_size) of
 	0.0 -> {none,none};
-	PtSize -> 
+	_PtSize -> 
 	    DynVs = sofs:from_external(lists:merge(Vs, StaticVs), [vertex]),
 	    SelVs = sofs:from_external(gb_sets:to_list(SelVs0), [vertex]),
 	    UnselDyn0 = case wings_pref:get_value(hide_sel_while_dragging) of
@@ -631,8 +648,6 @@ split_vs_dlist(Vs, StaticVs, {vertex,SelVs0}, #we{vp=Vtab}=We) ->
 	    UnselDyn = sofs:to_external(UnselDyn0),
 	    UnselDlist = gl:genLists(1),
 	    gl:newList(UnselDlist, ?GL_COMPILE),
-	    gl:pointSize(PtSize),
-	    gl:color3b(0, 0, 0),
 	    gl:'begin'(?GL_POINTS),
 	    List0 = sofs:from_external(gb_trees:to_list(Vtab), [{vertex,info}]),
 	    List1 = sofs:drestriction(List0, DynVs),
@@ -681,8 +696,6 @@ dynamic_vs(#dlo{src_we=#we{vp=Vtab},vs=[Static|_],
 		split=#split{dyn_vs=DynVs}}=D) ->
     UnselDlist = gl:genLists(1),
     gl:newList(UnselDlist, ?GL_COMPILE),
-    gl:pointSize(wings_pref:get_value(vertex_size)),
-    gl:color3b(0, 0, 0),
     gl:'begin'(?GL_POINTS),
     foreach(fun(V) ->
 		    gl:vertex3fv(gb_trees:get(V, Vtab))
