@@ -3,12 +3,12 @@
 %%
 %%     This module contains most edge command and edge utility functions.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson.
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson.
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_edge_cmd.erl 264 2008-02-28 15:50:13Z optigon $
+%%     $Id$
 %%
 
 -module(wings_edge_cmd).
@@ -21,15 +21,12 @@
 -include("wings.hrl").
 
 -import(lists, [foldl/3,mapfoldl/3,reverse/1,sort/1]).
--import(e3d_vec, [add/1,add/2,sub/2,neg/1,norm/1,len/1,
-		  average/1,average/2,
-		  dot/2,cross/2]).
+-import(e3d_vec, [add/2,sub/2,neg/1,norm/1,len/1,
+		  average/2, dot/2,cross/2]).
 
 menu(X, Y, St) ->
     Dir = wings_menu_util:directions(St),
-    Menu = [{basic,{?__(1,"Edge operations"),ignore}},
-	    {basic,separator},
-	    {?__(2,"Move"),{move,Dir},[],[magnet]},
+    Menu = [{?__(2,"Move"),{move,Dir},[],[magnet]},
 	    wings_menu_util:rotate(St),
 	    wings_menu_util:scale(St),
 	    {?__(3,"Slide"), slide,
@@ -37,6 +34,8 @@ menu(X, Y, St) ->
 	    separator,
 	    {?__(5,"Extrude"),{extrude,Dir}},
 	    separator,
+	    wings_menu_util:flatten(),
+	    separator,
 	    cut_line(St),
 	    {?__(6,"Connect"),connect,
 	     ?__(7,"Create a new edge by connecting midpoints of selected edges")},
@@ -44,7 +43,7 @@ menu(X, Y, St) ->
 	     ?__(9,"Round off selected edges")},
 	    separator,
 	    {?__(10,"Dissolve"), dslv(),
-		 {?__(11,"Eliminate selected edges"), "",
+		 {?__(11,"Eliminate selected edges"), [],
 		  ?__(50,"Eliminate selected edges and remove remaining isolated verts")},[]},
 		{?__(12,"Collapse"),collapse,
 	     ?__(13,"Delete edges, replacing them with vertices")},
@@ -113,6 +112,8 @@ command(bevel, St) ->
     ?SLOW(wings_extrude_edge:bevel(St));
 command({extrude,Type}, St) ->
     ?SLOW(wings_extrude_edge:extrude(Type, St));
+command({flatten,Plane}, St) ->
+    flatten(Plane, St);
 command(slide, St) ->
     slide(St);
 command(cut_pick, St) ->
@@ -176,7 +177,7 @@ cut_edges(Es, We) ->
 remove_nonconnectable([E|Es], Sel, We, Acc) ->
     Fs = wings_face:from_edges([E], We),
     NearEs = gb_sets:delete(E, gb_sets:from_ordset(wings_face:to_edges(Fs, We))),
-    case gb_sets:is_empty(gb_sets:intersection(Sel, NearEs)) of
+    case gb_sets:is_disjoint(Sel, NearEs) of
 	false ->
 	    remove_nonconnectable(Es, Sel, We, [E|Acc]);
 	true ->
@@ -610,7 +611,7 @@ loop_cut(Edges, #we{name=Name,id=Id,fs=Ftab}=We0, {Sel,St0}) ->
 	    %% We arbitrarily decide that the largest part of the object
 	    %% will be left unselected and will keep the name of the object.
 
-	    Parts1 = [{gb_trees:size(P),P} || P <- Parts0],
+	    Parts1 = [{gb_sets:size(P),P} || P <- Parts0],
 	    Parts2 = reverse(sort(Parts1)),
 	    [_|Parts] = [gb_sets:to_list(P) || {_,P} <- Parts2],
 
@@ -636,7 +637,7 @@ loop_cut_make_copies([P|Parts], We0, Sel0, #st{onext=Id}=St0) ->
 loop_cut_make_copies([], _, Sel, St) -> {Sel,St}.
 
 loop_cut_partition(Faces0, Edges, We, Acc) ->
-    case gb_trees:is_empty(Faces0) of
+    case gb_sets:is_empty(Faces0) of
 	true -> Acc;
 	false ->
 	    {AFace,Faces1} = gb_sets:take_smallest(Faces0),
@@ -671,3 +672,31 @@ collect_maybe_add(Work, Face, Edges, We, Res) ->
 		      end
 	      end
       end, Work, Face, We).
+
+%%%
+%%% The Flatten command.
+%%%
+
+flatten({'ASK',Ask}, St) ->
+    wings:ask(Ask, St, fun flatten/2);
+flatten({Plane,Center}, St) ->
+    flatten(Plane, Center, St);
+flatten(Plane, St) ->
+    flatten(Plane, average, St).
+
+flatten(Plane0, average, St) ->
+    Plane = wings_util:make_vector(Plane0),
+    {save_state,
+     wings_sel:map(
+       fun(Es, We) ->
+	       Vs = wings_edge:to_vertices(Es, We),
+	       wings_vertex:flatten(Vs, Plane, We)
+       end, St)};
+flatten(Plane0, Center, St) ->
+    Plane = wings_util:make_vector(Plane0),
+    {save_state,
+     wings_sel:map(
+       fun(Es, We) ->
+	       Vs = wings_edge:to_vertices(Es, We),
+	       wings_vertex:flatten(Vs, Plane, Center, We)
+       end, St)}.
