@@ -3,12 +3,12 @@
 %%
 %%     This module handles edge-loop commands.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_edge_loop.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_edge_loop).
@@ -19,7 +19,7 @@
 -export([edge_loop_vertices/2,edge_links/2,partition_edges/2]).
 
 -include("wings.hrl").
--import(lists, [sort/1,append/1,reverse/1,foldl/3]).
+-import(lists, [append/1,reverse/1,foldl/3]).
 
 %%%
 %%% Select next/previous edge loop.
@@ -186,7 +186,7 @@ next_edge(From, V, Face, Edge, Etab) ->
 
 add_mirror_edges(Edges, We) ->
     MirrorEdges = gb_sets:from_list(mirror_edges(We)),
-    case gb_sets:is_empty(gb_sets:intersection(Edges, MirrorEdges)) of
+    case gb_sets:is_disjoint(Edges, MirrorEdges) of
 	true -> Edges;
 	false -> gb_sets:union(Edges, MirrorEdges)
     end.
@@ -382,8 +382,8 @@ edge_loop_vertices1(Edges0, V, Vend, We, Acc) ->
     Res = wings_vertex:until(
 	    fun(Edge, _, Rec, A) ->
 		    case gb_sets:is_member(Edge, Edges0) of
-			true -> {Edge,wings_vertex:other(V, Rec)};
-			false -> A
+			false -> A;
+			true -> {Edge,wings_vertex:other(V, Rec)}
 		    end
 	    end, none, V, We),
     case Res of
