@@ -8,13 +8,13 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_ff_ndo.erl 179 2008-01-01 11:55:41Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_ff_ndo).
 -export([import/2,export/2]).
 -include("wings.hrl").
--import(lists, [sort/1,reverse/1,foldl/3,last/1,foreach/2]).
+-import(lists, [reverse/1,foldl/3,foreach/2]).
 
 -define(NDO_HEADER10, "nendo 1.0").
 -define(NDO_HEADER11, "nendo 1.1").
