@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_ff_wings.erl 179 2008-01-01 11:55:41Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_ff_wings).
@@ -358,12 +358,13 @@ share_list_2([{Vtab0,Etab0}|Ts],
     Etab = gb_trees:from_orddict(Etab0),
     We1 = wings_we:rebuild(We0#we{vp=Vtab,es=Etab,mat=default}),
     We2 = wings_facemat:assign(FaceMat, We1),
-    We = if
+    We3 = if
 	     NumHidden =:= 0 -> We2;
 	     true ->
 		 Hidden = lists:seq(0, NumHidden-1),
 		 wings_we:hide_faces(Hidden, We2)
 	 end,
+    We = ensure_valid_mirror_face(We3),
     share_list_2(Ts, Wes, [{Id,We}|Acc]);
 share_list_2([], [], Wes) -> sort(Wes).
 
@@ -433,6 +434,14 @@ share_tuple({A0,B0,C0}=Tuple0, Floats, Shared) ->
     end;
 share_tuple(none, _, Shared) -> {none,Shared}.
 
+%% Very old Wings files can have invalid mirror faces for some reason.
+ensure_valid_mirror_face(#we{mirror=none}=We) -> We;
+ensure_valid_mirror_face(#we{mirror=Face,fs=Ftab}=We) ->
+    case gb_trees:is_defined(Face, Ftab) of
+	false -> We#we{mirror=none};
+	true -> We
+    end.
+
 %%%
 %%% Import of old materials format (up to and including wings-0.94.02).
 %%%
@@ -476,9 +485,15 @@ export(Name, St0) ->
     wings_pb:start( ?__(1,"saving")),
     wings_pb:update(0.01, ?__(2,"lights")),
     Lights = wings_light:export(St0),
-    Materials = wings_material:used_materials(St0),
+    Materials = case wings_pref:get_value(save_unused_materials) of
+        true -> 
+            #st{mat=Mat} = St0,
+            gb_trees:to_list(Mat);
+        false -> 
+            wings_material:used_materials(St0)
+    end,
     #st{shapes=Shs0,views={CurrentView,_}} = St = 
-	remove_lights(St0),
+    remove_lights(St0),
     Sel0 = collect_sel(St),
     wings_pb:update(0.65, ?__(3,"renumbering")),
     {Shs1,Sel} = renumber(gb_trees:to_list(Shs0), Sel0, 0, [], []),
