@@ -3,22 +3,24 @@
 %%
 %%     This module implements the Help menu.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_help.erl 290 2008-03-09 18:02:40Z optigon $
+%%     $Id$
 %%
 
 -module(wings_help).
 -export([menu/1,command/2]).
--export([cmd/1,help_window/2]).
+-export([cmd/1,help_window/2,no_more_basic_menus/0]).
 
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
--import(lists, [reverse/1,foldl/3]).
+
+-import(erlang, [max/2]).
+-import(lists, [foldl/3]).
 
 menu(_) ->
     L = [getting_started,
@@ -30,7 +32,6 @@ menu(_) ->
 	 separator,
 	 lights,
 	 separator,
-	 advanced_menus,
 	 default_commands,
 	 separator,
 	 performance_tips,
@@ -55,8 +56,6 @@ header(defining_hotkeys) ->
     ?__(5,"How To Define Hotkeys");
 header(lights) ->
     ?__(6,"Light Basics");
-header(advanced_menus) ->
-    ?__(7,"Advanced Menus");
 header(default_commands) ->
     ?__(8,"Default Commands");
 header(performance_tips) ->
@@ -83,8 +82,6 @@ command_1(default_commands, Header) ->
     def_commands(Header);
 command_1(lights, Header) ->
     lights(Header);
-command_1(advanced_menus, Header) ->
-    advanced_menus(Header);
 command_1(performance_tips, Header) ->
     performance_tips(Header);
 command_1(opengl_info, Header) ->
@@ -92,6 +89,29 @@ command_1(opengl_info, Header) ->
 command_1(about, Header) ->
     about(Header).
 
+no_more_basic_menus() ->
+    Qs = {vframe,
+	  [{label,"You are seeing this message because the Advanced pop-up menus option was turned off in your preferences."},
+	   panel,
+	   {label,"From now on, Wings will only support Advanced pop-up menus."},
+	   panel,
+	   {label,"The major difference compared to the Basic pop-up menus, which you have been using, is that"},
+	   {label,"all three mouse buttons can be used to execute a command."},
+	   panel,
+	   {label,"Usually, the most common variation of a command is accessed by clicking the left mouse button,"},
+	   {label,"while the middle and right mouse buttons provide more advanced options for the command."},
+	   panel,
+	   {label,"Details about commands activated by each mouse button are displayed in the Information Line"},
+	   {label,"across the bottom of your screen when you hover over any menu item."},
+	   panel,
+	   {hframe,[{"Never show this message again",false,
+		     [{key,no_basic_menu_info}]},
+		    {button,ok,[ok]}]}]},
+    wings_ask:dialog("", Qs, fun([{K,V}]) ->
+				     wings_pref:set_value(K, V),
+				     ignore
+			     end).
+
 getting_started(Head) ->
     B = "(",
     E = ")",
@@ -116,7 +136,8 @@ getting_started(Head) ->
 
 	 use_one_or_two(),
 	 ?__(7,
-	      "Generally, L (left mouse button) is used for selecting and accepting, M (middle mouse button) for operating the camera, and R (right mouse button) to access the context-sensitive pop-up menus.")
+	      "Generally, L (left mouse button) is used for selecting and accepting, M (middle mouse button) for operating the camera, and R (right mouse button) to access the context-sensitive pop-up menus."),
+	 ?__(8, "Note that different mouse buttons can be used in pop-up menus to choose different variations of a command. Generally, the most common way to execute the command is found on the left mouse buttons, while the more advanced commands are found on the middle and right mouse buttons.")
 	],
     help_window(Head, H).
 
@@ -144,32 +165,6 @@ use_one_or_two() ->
 	edit_prefs()++
 	?__(2," dialog.").
 
-advanced_menus(Head) ->
-    Help = [?__(1,"In the ")++
-	    edit_prefs()++
-	    ?__(4,
-		 " dialog, there is a check box for \"Advanced Menus\"."),
-	    ?__(5,
-		 "Activating advanced menus provide the following additional features:"),
-	    ?__(6,"New commands: ")
-	    ++cmd([?__(7,"Face"),
-		   ?__(8,"Put On")
-		  ])++?__(9," and ")++
-	    cmd([?__(10,"Face"),
-		 ?__(11,"Lift")
-		]),
-	    ?__(12,"Vector based operations: The means to specify an axis (or vector) and to be able to re-locate it so it passes through a new point."),
-	    ?__(13,"Magnet operations: A facility that provides a way of attaining smooth modifications / transitions to surrounding geometry during the operation of any valid tool. Magnet type and influence radius settings provide further control options."),
-
-	    ?__(14,"Vector and magnet operations can be combined."),
-
-	    ?__(15,"With advanced menus turned on, many menu commands do different things depending on which mouse button you invoke them with."),
-
-	    ?__(16,"For instance, R clicking on the Rotate command allows you to specify an axis (or vector) to rotate your selection around, while M clicking lets you define an axis - and a new point through which that axis will pass. (A vector does not have to be parallel to an x, y or z axis.)"),
-
-	    ?__(17,"Reading the contents of the info line at the bottom of the Wings window is highly recommended when using advanced menus.")],
-    help_window(Head, Help).
-
 international(Head) ->
     Help = [?__(1,"Unfortunately, on French and German keyboards (and possibly others), the Undo/Redo commands will not be bound to the [Z] key. (That might be changed in a future release of Wings.)"),
 	    ?__(2,"On French keyboards, the Undo/Redo commands are found on the [W] key ([Ctrl]+[W], [Ctrl]+[Alt]+[W] and so on)."),
@@ -207,7 +202,11 @@ performance_tips(Head) ->
 	 B++?__(5,"type of graphics card"),
 	 B++?__(6,"amount of video memory"),
 	 B++?__(7,"the phase of the moon"),
+
 	 ?__(8,"Therefore, it is difficult to give any firm advice on how to improve Wings performance. The following tips MAY improve performance:"),
+
+	 B++?__(31,"Use fewer Undo levels. Setting to 4 or less (default 32) will let you work on models with almost 8 times the detail. In this case, using a shorter autosave period (default 10mins) may be advisable."),
+
 	 B++?__(9,"Try different number of colors and different screen resolutions. Especially if the graphics card doesn't have much memory, many colors and/or high resolution may drastically reduce performance. Using a smaller Wings window (not maximized) may also help."),
 
 	 B++?__(10,"Close unnecessary windows inside Wings."),
@@ -257,7 +256,7 @@ def_hotkeys(Head) ->
 		"press a mouse button to choose which variant of the command to "
 		"assign to the hotkey. (Note: The mouse cursor must remain inside "
 		"the menu when you click.)"),
-	    ?__(8,"4. The information line ask you to press the key that the command "
+	    ?__(8,"4. The information line asks you to press the key that the command "
 		"should be assigned to."),
 
  	    ?__(9,"To delete a hotkey, similarly high-light the command in a menu,"
@@ -513,12 +512,12 @@ splash_size(L) ->
     splash_size_1(L, 0, 0).
 
 splash_size_1([{icon,_,W,H}|T], W0, H0) ->
-    splash_size_1(T, wings_util:max(W, W0), H0+H);
+    splash_size_1(T, max(W, W0), H0+H);
 splash_size_1([{text,Text}|T], W0, H0) ->
     Tw = wings_text:width(Text),
-    splash_size_1(T, wings_util:max(W0, Tw), H0+wings_text:height()+4);
+    splash_size_1(T, max(W0, Tw), H0+wings_text:height()+4);
 splash_size_1([{spacer,W,H}|T], W0, H0) ->
-    splash_size_1(T, wings_util:max(W0, W), H0+H);
+    splash_size_1(T, max(W0, W), H0+H);
 splash_size_1([], W, H) -> {W,H}.
 
 draw_splash(L) ->
@@ -546,7 +545,7 @@ draw_splash_1([], _) -> ok.
     
 splash_contents() ->
     [{spacer,0,14},
-     {icon,wings,256,128},
+     {icon,about_wings,331,139},
      {text,[{bold,?WINGS_VERSION}]},
      {spacer,0,10},
      {text,?__(1,"Wings 3D is a subdivision modeler inspired")},
@@ -557,7 +556,7 @@ splash_contents() ->
      {text,?__(5,"(including commercial).")},
      {spacer,0,10},
      {text,?__(6,"Copyright") 
-      ++ [$\s,169] ++ " 2001-2005 "++"Bj" ++ [246] ++ "rn Gustavsson " ++
+      ++ [$\s,169] ++ " 2001-2009 "++"Bj" ++ [246] ++ "rn Gustavsson " ++
       ?__(7,"& Others")},
      {text,?__(8,"JPEG library: Copyright") ++ [$\s,169] ++
       " 1991-1998 Thomas G. Lane"}
