@@ -3,12 +3,12 @@
 %%
 %%     Initialization of Wings video and event handling.
 %%
-%%  Copyright (c) 2003-2005 Bjorn Gustavsson
+%%  Copyright (c) 2003-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_init.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_init).
@@ -18,6 +18,15 @@
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 
+-ifndef(SDL_GL_STEREO).
+%% Older ESDL.
+-define(SDL_GL_STEREO,             12).
+-define(SDL_GL_MULTISAMPLEBUFFERS, 13).
+-define(SDL_GL_MULTISAMPLESAMPLES, 14).
+-define(SDL_GL_ACCELERATED_VISUAL, 15).
+-define(SDL_GL_SWAP_CONTROL,       16).
+-endif.
+
 init() ->
     macosx_workaround(),
     os:putenv("SDL_HAS3BUTTONMOUSE", "true"),
@@ -25,14 +34,22 @@ init() ->
     wings_pref:set_default(window_size, {780,570}),
     TopSize = wings_pref:get_value(window_size),
     sdl:init(?SDL_INIT_VIDEO bor ?SDL_INIT_NOPARACHUTE),
-    Ebin = filename:dirname(code:which(?MODULE)),
-    IconFile = filename:join(Ebin, "wings.icon"),
-    catch sdl_video:wm_setIcon(sdl_video:loadBMP(IconFile), null),
+    set_icon(),
     sdl_video:gl_setAttribute(?SDL_GL_DOUBLEBUFFER, 1),
 
     %% Make sure that some video mode works. Otherwise crash early.
     %% From best to worst.
-    try_video_modes(opengl_modes(), TopSize),
+    try
+	sdl_video:gl_setAttribute(?SDL_GL_MULTISAMPLEBUFFERS, 1),
+	sdl_video:gl_setAttribute(?SDL_GL_MULTISAMPLESAMPLES, 4),
+	try_video_modes(opengl_modes(), TopSize)
+    catch
+	error:_ ->
+	io:fwrite("\nRetrying with multisampling disabled.\n"),
+	sdl_video:gl_setAttribute(?SDL_GL_MULTISAMPLEBUFFERS, 0),
+	sdl_video:gl_setAttribute(?SDL_GL_MULTISAMPLESAMPLES, 0),
+	try_video_modes(opengl_modes(), TopSize)
+    end,
     wings_gl:init_extensions(),
     wings_gl:init_restrictions(),
 
@@ -50,6 +67,44 @@ init() ->
 				 ?SDL_DEFAULT_REPEAT_INTERVAL),
     ok.
 
+set_icon() ->
+    Ebin = filename:dirname(code:which(?MODULE)),
+    IconFile = filename:join(Ebin,
+			     case os:type() of
+				 {unix,darwin} -> "wings_icon_big";
+				 _ -> "wings_icon_small"
+			     end),
+    set_icon_1(IconFile).
+
+set_icon_1(IconBase) ->
+    Bmp = sdl_video:loadBMP(IconBase ++ ".bmp"),
+    Mask = get_mask(IconBase ++ ".wbm"),
+    sdl_video:wm_setIcon(Bmp, Mask).
+
+%% get_mask(WBMFileName) -> Binary | null
+%%  Read a mask from a WBM file.
+%%  Wbmp format reference: http://en.wikipedia.org/wiki/Wbmp
+%%
+get_mask(IconBase) ->
+    case file:read_file(IconBase) of
+	{ok,Bin} -> get_mask_1(Bin);
+	{error,_} -> null
+    end.
+
+get_mask_1(<<0,0,T0/binary>>) ->
+    try
+	{_W,T} = get_uintvar(T0, 0),
+	{_H,Bits} = get_uintvar(T, 0),
+	Bits
+    catch _:_ ->
+	    null
+    end.
+
+get_uintvar(<<1:1,N:7,T/binary>>, Acc) ->
+    get_uintvar(T, (Acc bsl 7) bor N);
+get_uintvar(<<0:1,N:7,T/binary>>, Acc) ->
+    {(Acc bsl 7) bor N,T}.
+
 macosx_workaround() ->
     try 1.0/zero()
     catch
