@@ -3,12 +3,12 @@
 %%
 %%     This module contains most of the low-level GUI for Wings.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_io.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_io).
@@ -26,7 +26,7 @@
 	 draw_icons/1,draw_icon/3,draw_char/1,
 	 set_color/1]).
 -export([putback_event/1,putback_event_once/1,get_event/0,get_matching_events/1,
-	 set_timer/2,cancel_timer/1]).
+	 set_timer/2,cancel_timer/1,enter_event/1]).
 
 -export([reset_grab/0,grab/0,ungrab/2,is_grabbed/0,warp/2]).
 -export([ortho_setup/0,ortho_setup/1]).
@@ -35,8 +35,8 @@
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 
--import(lists, [flatmap/2,foldl/3,keysearch/3,member/2,
-		reverse/1,reverse/2,foreach/2,last/1]).
+-import(erlang, [max/2]).
+-import(lists, [flatmap/2,keysearch/3,member/2,reverse/1,reverse/2]).
 
 -define(ICON_WIDTH, 32).
 -define(ICON_HEIGHT, 28).
@@ -246,8 +246,8 @@ gradient_rect(X, Y, W, H=18, Color) ->
 	GreyValue = lists:nth(Idx+1, GradColors),
 	LineColor = mul_color(Color, GreyValue),
 	set_color(LineColor),
-	gl:vertex2f(X+W, Y+H-Idx),
-	gl:vertex2f(X,	 Y+H-Idx)
+	gl:vertex2f(X-0.5+W, Y-0.5+H-Idx),
+	gl:vertex2f(X-0.5,   Y-0.5+H-Idx)
 	end,
     gl:lineWidth(1),
     gl:'begin'(?GL_LINES),
@@ -462,7 +462,10 @@ create_textures_1([{Name,{3,W,H,Icon}}|T], Id, U, V, RowH0)
 create_textures_1(Icons, _, _, _, _) ->
     create_textures_2(Icons).
 
-create_textures_2([{Name,{Bpp,W,H,Icon}}|T]) ->
+create_textures_2([{Name,{Bpp,W,H,Icon0}}|T]) ->
+    TxW = max(nearest_power_of_two(W), W),
+    TxH = max(nearest_power_of_two(H), H),
+    Icon = pad_image(Icon0, W, TxW, H, TxH),
     [TxId] = gl:genTextures(1),
     gl:bindTexture(?GL_TEXTURE_2D, TxId),
     gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_MAG_FILTER, ?GL_LINEAR),
@@ -474,14 +477,35 @@ create_textures_2([{Name,{Bpp,W,H,Icon}}|T]) ->
 		 4 -> ?GL_RGBA
 	     end,
     gl:texImage2D(?GL_TEXTURE_2D, 0, Format,
-		  W, H, 0, Format, ?GL_UNSIGNED_BYTE, Icon),
-    [{Name,{TxId,W,H,0,0,1,1}}|create_textures_2(T)];
+		  TxW, TxH, 0, Format, ?GL_UNSIGNED_BYTE, Icon),
+    [{Name,{TxId,W,H,0,0,W/TxW,H/TxH}}|create_textures_2(T)];
 create_textures_2([]) -> [].
 
 div_uv(0, _) -> 0;
 div_uv(X, X) -> 1;
 div_uv(X, Y) -> X/Y.
 
+nearest_power_of_two(N) ->
+    nearest_power_of_two(N, 1).
+
+nearest_power_of_two(N, B) when N =< B -> B;
+nearest_power_of_two(N, B) -> nearest_power_of_two(N, B bsl 1).
+
+pad_image(Image, W, W, H, TxH) ->
+    pad_image_1(Image, H, TxH);
+pad_image(Image0, W, TxW, H, TxH) ->
+    Image = pad_image_hor(Image0, W, TxW, H),
+    pad_image_1(Image, H, TxH).
+
+pad_image_1(Image, H, TxH) ->
+    <<Image/binary,0:((byte_size(Image)*TxH) div H)/unit:8>>.
+
+pad_image_hor(Image, W, TxW, H) ->
+    OldRowSize = byte_size(Image) div H,
+    NewRowSize = (OldRowSize * TxW) div W,
+    Diff = NewRowSize - OldRowSize,
+    << <<Row/binary,0:Diff/unit:8>> || <<Row:OldRowSize/binary>> <= Image >>.
+
 create_buttons(Icons0) ->
     flatmap(fun({Name,{3,32,28,Icon}}) ->
 		    [{{Name,down},create_button(fun active/5, Icon)},
@@ -511,15 +535,15 @@ inactive(_X, _Y, R, G, B) -> [R,G,B].
 %%% Input.
 %%%
 
-putback_event(Event) ->
-    {In,Out} = get(?EVENT_QUEUE),
-    put(?EVENT_QUEUE, {In,[Event|Out]}).
+putback_event(Ev) ->
+    Q = get(?EVENT_QUEUE),
+    put(?EVENT_QUEUE, queue:in_r(Ev, Q)).
 
 putback_event_once(Ev) ->
-    {In,Out} = get(?EVENT_QUEUE),
-    case member(Ev, In) orelse member(Ev, Out) of
+    Q = get(?EVENT_QUEUE),
+    case queue:member(Ev, Q) of
 	true -> ok;
-	false -> put(?EVENT_QUEUE, {In,[Ev|Out]})
+	false -> put(?EVENT_QUEUE, queue:in_r(Ev, Q))
     end.
 
 get_event() ->
@@ -528,26 +552,21 @@ get_event() ->
 	Other -> Other
      end.
 
-get_matching_events(Filter) ->
+%% get_matching_events(FilterFun) -> [Event].
+%%       FilterFun(Element) -> true|false.
+%%  Remove from the queue and return in a list
+%%  all elements for which FilterFun(Element)
+%%  returns true. The elements in the returned
+%%  list will be in the same order as in the queue.
+%%
+get_matching_events(Filter) when is_function(Filter, 1) ->
     Eq = get(?EVENT_QUEUE),
-    get_matching_events_1(Filter, Eq, [], []).
-
-get_matching_events_1(Filter, Eq0, Match, NoMatch) ->
-    case queue:out(Eq0) of
-	{{value,Ev},Eq} ->
-	    case Filter(Ev) of
-		false ->
-		    get_matching_events_1(Filter, Eq, Match, [Ev|NoMatch]);
-		true ->
-		    get_matching_events_1(Filter, Eq, [Ev|Match], NoMatch)
-	    end;
-	{empty,{In,Out}} ->
-	    case Match of
-		[] -> [];
-		_ ->
-		    put(?EVENT_QUEUE, {In,reverse(NoMatch, Out)}),
-		    Match
-	    end
+    {Match,NoMatch} = lists:partition(Filter, queue:to_list(Eq)),
+    case Match of
+	[] -> [];
+	_ ->
+	    put(?EVENT_QUEUE, queue:from_list(NoMatch)),
+	    Match
     end.
 
 get_sdl_event() ->
@@ -556,6 +575,12 @@ get_sdl_event() ->
     put(?EVENT_QUEUE, Eq),
     Event.
 
+enter_event(Ev) ->
+    Eq0 = get(?EVENT_QUEUE),
+    Eq = queue:in(Ev, Eq0),
+    put(?EVENT_QUEUE, Eq),
+    ok.
+    
 read_events(Eq0) ->
     case sdl_events:peepEvents() of
 	[] -> read_out(Eq0);
