@@ -3,12 +3,12 @@
 %%
 %%     Implementation of lights.
 %%
-%%  Copyright (c) 2002-2005 Bjorn Gustavsson
+%%  Copyright (c) 2002-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_light.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_light).
@@ -59,9 +59,7 @@ menu(X, Y, St) ->
     NotAmb = {iff,[spot,infinite,point,area]},
     One = one_light,
     Dir = wings_menu_util:directions(St#st{selmode=body}),
-    Menu0 = [{basic,{?__(1,"Light operations"),ignore}},
-	     {basic,separator},
-	     {?__(2,"Move"),{move_light,Dir}},
+    Menu0 = [{?__(2,"Move"),{move_light,Dir}},
 	     {NotAmb,separator},
 	     {NotAmb,{?__(3,"Position Highlight"),
 		      {'VALUE',{position_highlight,{'ASK',{[point],[]}}}},
@@ -725,9 +723,7 @@ camera_infinite_2_1() ->
 	  }.
 
 
-infinite({X,Y,Z}) ->
-    {X,Y,Z,0.0};
-infinite(V) -> V.
+infinite({X,Y,Z}) -> {X,Y,Z,0.0}.
 
 modeling_lights(global, _Type) -> ok;
 modeling_lights(camera, Type) ->
