@@ -3,33 +3,50 @@
 %%
 %%     This module implements the Magnet command.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_magnet.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_magnet).
 -export([setup/3,transform/2,recalc/3,flags/2,dialog/1,dialog/2,
-	 info_string/0]).
+	 info_string/0,drag_help/1,hotkey/1]).
 
 -include("wings.hrl").
--import(lists, [map/2,foldr/3,foldl/3,sort/1,concat/1,reverse/1]).
+-import(lists, [foldl/3]).
 -define(HUGE, 9.9E307).
 
 setup({magnet,Type,Route,Point}, VsSel, We) ->
-    {VsDist,R} = case Route of
+    {VsDist0,R} = case Route of
 		     shortest -> inf_shortest(Point, VsSel, We);
 		     midpoint -> inf_midpoint(Point, VsSel, We);
 		     surface -> inf_surface(Point, VsSel, We)
 		 end,
+    VsDist = minus_locked_vs(VsSel,VsDist0,We),
     Magnet = {Type,R},
     VsInf = recalc(1.0, VsDist, Magnet),
     Affected = foldl(fun({V,_,_,_}, A) -> [V|A] end, [], VsInf),
     {VsInf,Magnet,Affected}.
 
+%%%% Requires the wpc_magnet_mask plugin.
+minus_locked_vs(VsSel,VsDistList,#we{pst=Pst}) ->
+    Mask = wings_pref:get_value(magnet_mask_on),
+    case gb_trees:is_defined(wpc_magnet_mask,Pst) of
+      true when Mask ->
+        LockedVs = gb_sets:to_list(wpc_magnet_mask:get_locked_vs(Pst)),
+        Unaffected = LockedVs -- VsSel,
+		remove_masked(Unaffected, VsDistList);
+      _otherwise ->
+        VsDistList
+    end.
+
+remove_masked([V|Unaffected],VsDistList) ->
+    remove_masked(Unaffected,lists:keydelete(V,1,VsDistList));
+remove_masked([],VsDistList) -> VsDistList.
+
 info_string() ->
     ["(",?__(1,"Magnet route:"),
      magnet_route(wings_pref:get_value(magnet_distance_route)),
