@@ -3,12 +3,12 @@
 %%
 %%     Maintains the vertex palette window.
 %%
-%%  Copyright (c) 2004-2008 Bjorn Gustavsson, Dan Gudmundsson
+%%  Copyright (c) 2004-2009 Bjorn Gustavsson, Dan Gudmundsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_palette.erl 207 2008-01-04 16:18:30Z bjorng $
+%%     $Id$
 %%
 -module(wings_palette).
 
@@ -19,8 +19,7 @@
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 
--import(lists, [map/2,foldl/3,reverse/1,reverse/2,
-		keymember/3,keysearch/3,keydelete/3,sort/1]).
+-import(lists, [foldl/3,reverse/1]).
 
 -define(BOX_W, 16).
 -define(BOX_H, 16).
@@ -152,7 +151,7 @@ event(got_focus, _) ->
     CL = [ModName,$+,wings_msg:button_format(?__(4,"Clear color"))],
     Msg = wings_msg:join([L,CL,MR]),
     wings_wm:message(Msg),
-    wings_wm:dirty();
+    keep;
 
 event({current_state,St = #st{pal=StPal}}, Pst=#pst{w=W,h=H}) ->
     case StPal of
