@@ -3,26 +3,25 @@
 %%
 %%     This module handles picking using OpenGL.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_pick.erl 292 2008-03-14 00:22:28Z optigon $
+%%     $Id$
 %%
 
 -module(wings_pick).
 -export([event/2,event/3,hilite_event/3, hilite_event/4]).
 -export([do_pick/3]).
-
+-export([marquee_pick/3,paint_pick/3]).
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 -include("e3d.hrl").
 
--import(lists, [foreach/2,last/1,reverse/1,reverse/2,
-		sort/1,foldl/3,map/2,min/1,
-		keysearch/3,member/2,delete/2,seq/2]).
+-import(lists, [foreach/2,reverse/2,sort/1,map/2,min/1,
+		keysearch/3,member/2]).
 
 %% For ordinary picking.
 -record(pick,
@@ -54,8 +53,11 @@ event(#mousemotion{}=Mm, #st{selmode=Mode}=St, Redraw) ->
 	false -> next;
 	true -> {seq,push,handle_hilite_event(Mm, #hl{st=St,redraw=Redraw})}
     end;
-event(#mousebutton{button=1,x=X,y=Y,mod=Mod,state=?SDL_PRESSED}, St, _) ->
-    pick(X, Y, Mod, St);
+event(#mousebutton{button=1,x=X,y=Y,mod=Mod,state=?SDL_PRESSED}, St, _)
+        when Mod band (?SHIFT_BITS bor ?CTRL_BITS) =/= 0 ->
+    marquee_pick(X, Y, St);
+event(#mousebutton{button=1,x=X,y=Y,state=?SDL_PRESSED}, St, _) ->
+    paint_pick(X, Y, St);
 event(_, _, _) -> next.
 
 hilite_event(Mm, St, Redraw) ->
@@ -76,15 +78,16 @@ hilite_enabled(edge) -> wings_pref:get_value(edge_hilite);
 hilite_enabled(face) -> wings_pref:get_value(face_hilite);
 hilite_enabled(body) -> wings_pref:get_value(body_hilite).
 
-pick(X, Y, Mod, St) when Mod band (?SHIFT_BITS bor ?CTRL_BITS) =/= 0 ->
+marquee_pick(X, Y, St)  ->
     Pick = #marquee{ox=X,oy=Y,st=St},
-    clear_hilite_marquee_mode(Pick);
-pick(X, Y, _, St0) ->
+    clear_hilite_marquee_mode(Pick).
+paint_pick(X, Y, St0) ->
     case do_pick(X, Y, St0) of
 	none ->
 	    Pick = #marquee{ox=X,oy=Y,st=St0},
 	    clear_hilite_marquee_mode(Pick);
 	{PickOp,_,St} ->
+	    wings_wm:grab_focus(),
 	    wings_wm:dirty(),
 	    wings_draw:refresh_dlists(St),
 	    Pick = #pick{st=St,op=PickOp},
@@ -255,7 +258,7 @@ enhanced_hl_info(Base,#hl{redraw=#st{sel=[],shapes=Shs},prev={SelMode,_,{Obj,Ele
             Length = e3d_vec:dist({Xa,Ya,Za}, {Xb,Yb,Zb}),
             {X,Y,Z} = e3d_vec:average({Xa,Ya,Za}, {Xb,Yb,Zb}),
             [Base|io_lib:format(?__(3,". Midpoint <~s  ~s  ~s>\nLength ~s") ++
-                                "  <~s  ~s  ~s>",
+                                "  <~s  ~s  ~s>", %++ "\nVa = ~p\nVb = ~p",
                                 [wings_util:nice_float(X),
                                  wings_util:nice_float(Y),
                                  wings_util:nice_float(Z),
@@ -263,6 +266,7 @@ enhanced_hl_info(Base,#hl{redraw=#st{sel=[],shapes=Shs},prev={SelMode,_,{Obj,Ele
                                  wings_util:nice_float(abs(Xb - Xa)),
                                  wings_util:nice_float(abs(Yb - Ya)),
                                  wings_util:nice_float(abs(Zb - Za))])];
+								% Va,Vb])];
           face ->
             {X,Y,Z} = wings_face:center(Elem, We),
             Area = area_info(Elem, We),
@@ -331,7 +335,11 @@ marquee_event(#mousemotion{x=X,y=Y}, #marquee{cx=Cx,cy=Cy}=M) ->
     draw_marquee(Cx, Cy, M),
     draw_marquee(X, Y, M),
     get_marquee_event(M#marquee{cx=X,cy=Y});
-marquee_event(#mousebutton{x=X0,y=Y0,mod=Mod,button=1,state=?SDL_RELEASED}, M) ->
+marquee_event(#mousebutton{x=X0,y=Y0,mod=Mod,button=B,state=?SDL_RELEASED}, M)
+        when B==1;B==2 ->
+    %% Button 2 is only used in Tweak Mode for Maya cam. If there are any
+    %% issues I will revert the code to only accept release states from B 1.
+    %% - Richard
     {Inside,Op} =
 	if
 	    Mod band ?SHIFT_BITS =/= 0, Mod band ?CTRL_BITS =/= 0 ->
@@ -499,15 +507,38 @@ pick_event(#mousemotion{x=X,y=Y}, #pick{op=Op,st=St0}=Pick) ->
 	    get_pick_event(Pick#pick{st=St});
 	{_,_,_} -> keep
     end;
-pick_event(#mousebutton{button=1,state=?SDL_RELEASED}, #pick{st=St}) ->
+pick_event(#mousebutton{button=B,state=?SDL_RELEASED}, #pick{st=St})
+        when B==1;B==2 ->
+    %% Button 2 is only used in Tweak Mode for Maya cam. If there are any
+    %% issues I will revert the code to only accept release states from B 1.
+    %% - Richard
+    wings_wm:release_focus(),
     wings_wm:later({new_state,St}),
     pop;
 pick_event(_, _) -> keep.
 
 do_pick(X, Y, St) ->
     case raw_pick(X, Y, St) of
-	none -> none;
-	Hit -> update_selection(Hit, St)
+	none ->
+	    none;
+	Hit ->	    % Hit is {edge,original,{1,1}}
+	    %Mod = sdl_keyboard:getModState(),
+	    %LKey = Mod band ?KMOD_LALT =/= 0,
+	    %RKey = Mod band ?KMOD_RALT =/= 0,
+	    %% LKey = Mod band ?KMOD_LMETA =/= 0,
+	    %% RKey = Mod band ?KMOD_RMETA =/= 0,
+	    %if
+		%LKey ->
+		%    {_,_,St2} = update_selection(Hit, St),
+		%    St3 = wings_edge_loop:select_loop(St2),
+		%    {add,original,St3};
+		%RKey ->
+		%    {_,_,St2} = update_selection(Hit, St),
+		%    St3 = wings_edge:select_edge_ring(St2),
+		%    {add,original,St3};
+		%true ->
+		    update_selection(Hit, St)
+	    %end
     end.
 
 raw_pick(X0, Y0, #st{selmode=Mode}=St) ->
