@@ -4,12 +4,12 @@
 %%
 %%     Preference management.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_pref_dlg.erl 309 2008-04-01 20:01:15Z optigon $
+%%     $Id$
 %%
 
 -module(wings_pref_dlg).
@@ -18,7 +18,7 @@
 -include("wings.hrl").
 -import(wings_pref, [get_value/1,set_value/2]).
 
--import(lists, [foreach/2,keysearch/3,map/2,reverse/1,sort/1,last/1]).
+-import(lists, [reverse/1]).
 -define(NONZERO,1.0e-6).
 
 menu(_St) ->
@@ -57,7 +57,13 @@ gen_prefs() ->
 	   [{?__(1,"Unselected Size"),vertex_size,
 	     [{info,?__(2,"Size in pixels of unselected vertices (0.0 means not shown)")}]},
 	    {?__(3,"Selected Size"),selected_vertex_size,
-	     [{info,?__(4,"Size in pixels of selected vertices")}]}]}],
+	     [{info,?__(4,"Size in pixels of selected vertices")}]},
+	    {?__(65,"Magnet Mask Vertex Size"),masked_vertex_size,
+	     [{info,?__(66,"Size in pixels of magnet mask vertices")}]}]},
+	    {hframe,[{label_column,[{color,?__(67,"Unselected Vertices"),vertex_color,
+	     [{info,?__(68,"Color of unselected vertices")}]}]},
+	    {label_column,[{color,?__(69,"Magnet Mask"),masked_vertex_color,
+	     [{info,?__(70,"Color of magnet mask vertices")}]}]}]}],
 	 [{title,?__(5,"Vertex Display")}]},
 	{vframe,
 	 [{label_column,
@@ -146,21 +152,36 @@ gen_prefs() ->
 	   [{info,?__(34,"Always show the grid when the view is aligned along one of the major axes")}]},
 	    {?__(aa_ortho1,"Force Axis-Aligned Ortho"),force_ortho_along_axis,
 	   [{info, ?__(aa_ortho2,"Always go into orthogonal mode when the view is aligned along one of the major axes")}]}],
-     [{title,?__(35,"Grid")}]}]}.
+     [{title,?__(35,"Grid")}]},
+     
+  {hframe,
+	 [{"Enable Multi-Sampling",multisample, 
+	  [{info, "Anti-Alias the whole interface, including polygons."}]}],
+     [{title,"Anti-Aliasing"}]}]}.
 
 advanced_prefs() ->
-    DisableHook = fun (is_disabled, {_Var,_I,Store}) ->
-			  not gb_trees:get(advanced_menus, Store);
-		      (_, _) -> void
-		  end,
+%%     DisableHook = fun (is_disabled, {_Var,_I,Store}) ->
+%% 			  not gb_trees:get(advanced_menus, Store);
+%% 		      (_, _) -> void
+%% 		  end,
     SuperDisable = fun (is_disabled, {_Var,_I,Store}) ->
-			   not gb_trees:get(advanced_menus, Store) orelse
+%%			   not gb_trees:get(advanced_menus, Store) orelse
 			       not gb_trees:get(use_temp_sel, Store);
 		       (_, _) ->	void
 		   end,
-    Flags = [{hook,DisableHook}],
+    HighlightDisable = fun (is_disabled, {_Var,_I,Store}) ->
+		       not gb_trees:get(use_temp_sel, Store);
+		       (_, _) ->	void
+		   end,
+%%    Flags = [{hook,DisableHook}],
+    Flags = [],
+    Disable = fun (is_disabled, {_Var,_I, Store}) ->
+		       not gb_trees:get(drag_custom,Store);
+		       (_, _) -> void
+		       end,
 
-    {vframe,
+    {hframe,
+    [{vframe,
      [{?__(1,"Default Commands"),default_commands,
        [{info,?__(2,"Allow defining commands that can be invoked by Ctrl+L or Ctrl+M")}]},
       {?__(3,"Use Highlight as Temporary Selection"),use_temp_sel,
@@ -171,13 +192,17 @@ advanced_prefs() ->
       {?__(8,"Hide Selection While Moving Camera"),hide_sel_in_camera_moves,
        [{info,?__(9,"Don't show the selection when the camera is being moved")}]},
       panel,
-      {?__(10,"Advanced Menus"),advanced_menus,
-       [{info,?__(11,"More commands and more options, such as magnets")}]},
+%%      {?__(10,"Advanced Menus"),advanced_menus,
+%%       [{info,?__(11,"More commands and more options, such as magnets")}]},
       {?__(21,"Use Mirror for Selections"),use_mirror_for_sels,
        [{info,?__(22,"Default to using the virtual mirror for secondary selections")}]},
       {?__(12,"Power-user temporary selections"),use_super_temp_sel,
        [{info,?__(13,"In the secondary selection mode, RMB-clicking always adds to the selection")},{hook,SuperDisable}]},
-      {vframe,
+      {?__(54,"Extend selection via hotkey on temporary highlight"),hilite_select,
+	   [{info,?__(55,"Affects: Select All, Edge Loop, Edge Ring, Select Similar, Similar Normals, Similar Materials, and Similar Area.")},
+	     {hook,HighlightDisable}]},
+      {?__(53,"Launch Wings in Tweak Mode"),start_in_tweak},
+     {vframe,
        [{label_column,
      [{?__(14,"Length"),active_vector_size,
        [{info,?__(15,"Length of vector in secondary selections")},{range,{0.1,10.0}}|Flags]},
@@ -188,26 +213,46 @@ advanced_prefs() ->
        [{title,?__(20,"Vector Display")}]},
       {vframe,
        [{label_column,
-	 [{?__(23,"Size"),clip_plane_size,
-	   [{info,?__(24,"Size of user-defined clipping indicator")},{range,{0.1,100.0}}|Flags]},
-	  {color,?__(25,"Color"),clip_plane_color,
-	   [{info,?__(26,"Color of user-defined clipping indicator")}]}]}],
-       [{title,?__(27,"Clipping Plane Indicator")}]}]}.
+     [{?__(23,"Size"),clip_plane_size,
+       [{info,?__(24,"Size of user-defined clipping indicator")},{range,{0.1,100.0}}|Flags]},
+      {color,?__(25,"Color"),clip_plane_color,
+       [{info,?__(26,"Color of user-defined clipping indicator")}]}]}],
+       [{title,?__(27,"Clipping Plane Indicator")}]},
+     {vframe,
+       [{?__(28,"Selected Geometry"),highlight_aim_at_selected},
+        {?__(29,"Unselected Geometry"),highlight_aim_at_unselected}],
+       [{title,?__(30,"Highlight Aim Targets")},{hook,HighlightDisable}]}]},
+     {vframe,
+        [{vframe,
+          [{?__(36,"Switching Modes Resets Model's Position"),drag_resets,
+              [{info,?__(37,"Only affects tools with more than one drag mode such as Edge|Slide")}]},
+           {?__(38,"Customize Drag Response"),drag_custom,
+               [{info,?__(39,"Customize the drag to mouse motion ratio. Unchecked uses default settings.")}]}]},
+        {vframe,
+          [{?__(40,"Factor in Distance from Camera"),drag_cam_dist_abs},
+           {hframe,
+              [{slider,{text,drag_speed_abs,[{range,{1.0,10.0}}]}}]}],
+          [{title,?__(50,"Mouse Speed for Dragging")},{hook,Disable}]},
+         {vframe,
+          [{?__(43,"Factor in Distance from Camera"),drag_cam_dist_relative},
+            {hframe,
+              [{slider,{text,drag_speed_relative,[{range,{1.0,10.0}}]}}]}],
+           [{title,?__(51,"Mouse Speed for Scaling")},{hook,Disable}]},
+         {vframe,
+          [{hframe,
+           [{slider,{text,drag_speed_rotate,[{range,{1.0,10.0}}]}}]}],
+       [{title,?__(52,"Mouse Speed for Rotating")},{hook,Disable}]}],
+    [{title,?__(49,"Drag Preferences")}]}]}.
 
 constraint_prefs() ->
     A = wings_s:modkey(alt),
     S = wings_s:modkey(shift),
-	C = wings_s:modkey(ctrl),
-	P = "+",
-	CS = C++P++S,
-	CA = C++P++A,
-	SA = S++P++A,
-   CSA = C++P++S++P++A,
-	
-	Disable = fun (is_disabled, {_Var,_I, Store}) ->
-       not gb_trees:get(drag_custom,Store);
-       (_, _) -> void
-       end,
+    C = wings_s:modkey(ctrl),
+    P = "+",
+    CS = C++P++S,
+    CA = C++P++A,
+    SA = S++P++A,
+    CSA = C++P++S++P++A,
 
     Set_Con_Default = [{A,alt},
                        {S,shift},
@@ -222,29 +267,29 @@ constraint_prefs() ->
      [{hframe,
        [{vframe,
       [{vframe,
-	 [{label_column,
-	   [{A,con_dist_alt,
-	     [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
+     [{label_column,
+       [{A,con_dist_alt,
+         [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
           [A])},{range,{?NONZERO,1000.0,10.0}}]},
-	    {S,con_dist_shift,
-	     [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
+        {S,con_dist_shift,
+         [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
           [S])},{range,{?NONZERO,1000.0,1.0}}]},
-	    {C,con_dist_ctrl,
-	     [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
+        {C,con_dist_ctrl,
+         [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
           [C])},{range,{?NONZERO,1000.0,0.1}}]},
-	    {CS,con_dist_ctrl_shift,
-	     [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
+        {CS,con_dist_ctrl_shift,
+         [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
           [CS])},{range,{?NONZERO,1000.0,0.01}}]},
-	    {CA,con_dist_ctrl_alt,
-	     [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
+        {CA,con_dist_ctrl_alt,
+         [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
           [CA])},{range,{?NONZERO,1000.0,0.001}}]},
-	    {SA,con_dist_shift_alt,
-	     [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
+        {SA,con_dist_shift_alt,
+         [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
           [SA])},{range,{?NONZERO,1000.0,0.25}}]},
-	    {CSA,con_dist_ctrl_shift_alt,
-	     [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
+        {CSA,con_dist_ctrl_shift_alt,
+         [{info,wings_util:format(?__(12,"Constraint bound to ~s for distance operations."),
           [CSA])},{range,{?NONZERO,1000.0,0.025}}]}]}],
-	 [{title,?__(22,"Distance Constraints")}]},
+     [{title,?__(22,"Distance Constraints")}]},
 
     {vframe,
      [{label_column,
@@ -338,31 +383,7 @@ constraint_prefs() ->
        [{title,?__(23,"Rotation Constraints")}]}]}]}]},
       {vframe,[{label,?__(30,"Deleting value or entering '0' resets constraint to Wings default")}]}],
     [{title,?__(32,"Default Constraint Values")}]},
-
-      {hframe,
-       [{vframe,
-        [{vframe,
-          [{?__(36,"Switching Modes Resets Model's Position"),drag_resets,
-              [{info,?__(37,"Only affects tools with more than one drag mode such as Edge|Slide")}]},
-           {?__(38,"Customize Drag Response"),drag_custom,
-               [{info,?__(39,"Customize the drag to mouse motion ratio. Unchecked uses default settings.")}]}]},
-	{vframe,
-          [{?__(40,"Factor in Distance from Camera"),drag_cam_dist_abs},
-           {hframe,
-              [{slider,{text,drag_speed_abs,[{range,{1.0,10.0}}]}}]}],
-          [{title,?__(42,"Absolute Speed")},{hook,Disable}]},
-         {vframe,
-          [{?__(43,"Factor in Distance from Camera"),drag_cam_dist_relative},
-            {hframe,
-              [{slider,{text,drag_speed_relative,[{range,{1.0,10.0}}]}}]}],
-           [{title,?__(46,"Relative Speed")},{hook,Disable}]},
-         {vframe,
-          [{hframe,
-           [{slider,{text,drag_speed_rotate,[{range,{1.0,10.0}}]}}]}],
-       [{title,?__(48,"Rotation Speed")},{hook,Disable}]}],
-    [{title,?__(49,"Drag Preferences")}]},
-		
-    {vframe,
+    {hframe,
      [{hframe,
        [{vframe,
         [{label,?__(1,"Distance")},
@@ -373,15 +394,16 @@ constraint_prefs() ->
          {menu,Set_Con_Default,con_rot_default},
          {menu,Set_Con_Default,con_scale_default}]}],
      [{title,?__(4,"Default Modifier Keys")}]},
-       {hframe,
+      {vframe,
+       [{vframe,
         [{vradio,[{?__(9,"Distance Constraints"),con_dist_},
           % ends with _ to allow atom_to_string completion
           {?__(10,"Alternate Distances"),con_dist_a_}], con_dist_set}],
        [{title,?__(11,"Save Constraints to...")}]},
        {?__(33,"Use Alternate Constraints"),con_alternate,
          [{info,?__(34,"Checked uses alternate distances, reciprocal scale factors, and supplementary angles.")
-         ++?__(35," Switch on the fly pressing [Shift]+[Tab] during a drag operation.")}]}],
-    [{title,?__(31,"Set Constraint Preferences")}]}]}]}.
+         ++?__(35," Switch on the fly pressing [Shift]+[Tab] during a drag operation.")}]}]}],
+    [{title,?__(31,"Set Constraint Preferences")}]}]}.
 
 ui_prefs() ->
     Fonts = wings_text:fonts(),
@@ -398,29 +420,33 @@ ui_prefs() ->
 	   [{title,?__(1,"System Font")}]},
 	  {hframe,
 	   [{vframe,
-	     [{label,?__(2,"Desktop/Geometry Background")},
+	     [{label,?__(12,"Title (Active) Background")},
+	      {label,?__(11,"Title (Passive) Background")},
+	      {label,?__(10,"Title Text")},
+	      {label,?__(2,"Desktop/Geometry Background")},
+	      {label,?__(28,"Menu Bar Background")},
+	      {label,?__(6,"Menu Background")},
 	      {label,?__(3,"Menu Text")},
 	      {label,?__(4,"Menu Highlight")},
 	      {label,?__(5,"Menu Highlighted Text")},
-	      {label,?__(6,"Menu Background")},
+	      {label,?__(9,"Dialog Background")},
 	      {label,?__(7,"Dialog Text")},
 	      {label,?__(8,"Dialog (Disabled) Text")},
-	      {label,?__(9,"Dialog Background")},
-	      {label,?__(10,"Title Text")},
-	      {label,?__(11,"Title (Passive) Background")},
-	      {label,?__(12,"Title (Active) Background")}]},
+	      {label,?__(29,"Info Line Background")}]},
 	    {vframe,
-	     [{color,background_color},
+	     [{color,title_active_color},
+	      {color,title_passive_color},
+	      {color,title_text_color},
+	      {color,background_color},
+	      {color,menu_bar_bg},
+	      {color,menu_color},
 	      {color,menu_text},
 	      {color,menu_hilite},
 	      {color,menu_hilited_text},
-	      {color,menu_color},
+	      {color,dialog_color},
 	      {color,dialog_text},
 	      {color,dialog_disabled},
-	      {color,dialog_color},
-	      {color,title_text_color},
-	      {color,title_passive_color},
-	      {color,title_active_color}]}],
+	      {color,info_line_bg}]}],
 	   [{title,?__(13,"Colors")}]}
 	 ]},
 	{vframe,
@@ -481,6 +507,7 @@ language_name("de") -> "Deutsch";
 language_name("es") -> "Espaol";
 language_name("fi") -> "Suomi";
 language_name("fr") -> "Franais";
+language_name("hu") -> "Hungarian";
 language_name("it") -> "Italiano";
 language_name("pl") -> "Polski";
 language_name("pt") -> "Portugus";
@@ -509,7 +536,7 @@ misc_prefs() ->
 	       {text,autosave_time,[{hook,AutoFun},{range,{1,1440}}]},
 	       {label,?__(3,"minutes")}]},
       {hframe,[{label,?__(4,"Undo levels")},
-	       {text,num_undo_levels,[{range,{10,128}}]}]},
+	       {text,num_undo_levels,[{range,{1,128}}]}]},
       {vframe,
        [{label_column,
 	 [{?__(5,"Angle"),auto_rotate_angle},
