@@ -4,18 +4,20 @@
 %%     Render all objects and helpers (such as axes) in the scene.
 %%     Used for the Geometry and AutoUV windows.
 %%
-%%  Copyright (c) 2001-2006 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_render.erl 235 2008-01-24 08:30:08Z optigon $
+%%     $Id$
 %%
 
 -module(wings_render).
 -export([init/0, render/1,polygonOffset/1,
 	 enable_lighting/0, disable_lighting/0]).
 
+-import(erlang, [max/2]).
+
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 
@@ -45,6 +47,14 @@ render(#st{selmode=Mode}=St) ->
 		  ?GL_LIGHTING_BIT),
     gl:enable(?GL_DEPTH_TEST),
     gl:enable(?GL_CULL_FACE),
+    case wings_pref:get_value(multisample) of
+	true ->
+	    gl:enable(?GL_MULTISAMPLE);
+	false ->
+	    gl:disable(?GL_MULTISAMPLE);
+	undefined ->
+	    wings_pref:set_default(multisample, true)
+    end,
     wings_view:load_matrices(true),
     ground_and_axes(),
     mini_axis_icon(),
@@ -192,7 +202,7 @@ render_plain(#dlo{src_we=We}=D, SelMode) ->
     wings_proxy:draw(D, Wire),
     render_plain_rest(D, Wire, SelMode).
 
-render_plain_rest(D, Wire, SelMode) ->
+render_plain_rest(#dlo{}=D, Wire, SelMode) ->
     gl:disable(?GL_POLYGON_OFFSET_LINE),
     gl:disable(?GL_POLYGON_OFFSET_FILL),
 
@@ -200,16 +210,17 @@ render_plain_rest(D, Wire, SelMode) ->
     case Wire of
 	true ->
 	    gl:disable(?GL_CULL_FACE),
-	    draw_orig_sel(D),
 	    draw_sel(D),
+	    draw_orig_sel(D),
 	    gl:enable(?GL_CULL_FACE);
 	false ->
-	    draw_orig_sel(D),
-	    draw_sel(D)
+	    draw_sel(D),
+	    draw_orig_sel(D)
     end,
     draw_vertices(D, SelMode),
     draw_hard_edges(D, SelMode),
-    draw_normals(D).
+    draw_normals(D),
+	draw_plugins(plain,D,SelMode). %% arbitrary placement in the grand scheme of things
 
 render_smooth(#dlo{work=Work,edges=Edges,smooth=Smooth,transparent=Trans,
 		   src_we=We,proxy_data=Pd,open=Open}=D,
@@ -271,8 +282,9 @@ render_smooth(#dlo{work=Work,edges=Edges,smooth=Smooth,transparent=Trans,
 	false -> ok
     end,
     draw_hilite(D),
+    draw_sel(D),
     draw_orig_sel(D),
-    draw_sel(D).
+	draw_plugins(smooth,D,none).
 
 wire(#we{id=Id}) ->
     W = wings_wm:get_prop(wireframed_objects),
@@ -304,10 +316,15 @@ sel_color() ->
     gl:color3fv(wings_pref:get_value(selected_color)).
 
 draw_vertices(#dlo{src_we=#we{perm=P},vs=VsDlist}, vertex) when ?IS_SELECTABLE(P) ->
+    {R,G,B} = wings_pref:get_value(vertex_color),
+	Size = wings_pref:get_value(vertex_size),
+    gl:pointSize(Size),
+    gl:color3f(R,G,B),
     wings_dl:call(VsDlist);
 draw_vertices(_, _) -> ok.
 
-draw_hilite(#dlo{hilite=DL}) -> wings_dl:call(DL).
+draw_hilite(#dlo{hilite=DL}) -> 
+    wings_dl:call(DL).
 
 draw_orig_sel(#dlo{orig_sel=none}) -> ok;
 draw_orig_sel(#dlo{orig_sel=Dlist,orig_mode=Mode}) ->
@@ -343,7 +360,7 @@ draw_hard_edges(#dlo{hard=Hard}, SelMode) ->
     gl:lineWidth(hard_edge_width(SelMode)),
     gl:color3fv(wings_pref:get_value(hard_edge_color)),
     wings_dl:call(Hard).
-
+	
 draw_normals(#dlo{normals=none}) -> ok;
 draw_normals(#dlo{normals=Ns}) ->
     gl:color3f(0, 0, 1),
@@ -352,8 +369,12 @@ draw_normals(#dlo{normals=Ns}) ->
 
 edge_width(edge) -> wings_pref:get_value(edge_width);
 edge_width(_) -> 1.
+
 hard_edge_width(edge) -> wings_pref:get_value(hard_edge_width);
-hard_edge_width(_) -> wings_pref:get_value(hard_edge_width) -1.
+hard_edge_width(_) -> max(wings_pref:get_value(hard_edge_width) - 1, 1).
+
+draw_plugins(Flag,D,Selmode) ->
+    wings_plugin:draw(Flag, D, Selmode).
 
 ground_and_axes() ->
     Axes = wings_wm:get_prop(show_axes),
