@@ -3,12 +3,12 @@
 %%
 %%     This module contains utility functions for vertices.
 %%
-%%  Copyright (c) 2001-2008 Bjorn Gustavsson
+%%  Copyright (c) 2001-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_vertex.erl 293 2008-03-21 09:57:09Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_vertex).
@@ -25,7 +25,7 @@
 	 isolated/1,edge_through/3,edge_through/4]).
 
 -include("wings.hrl").
--import(lists, [member/2,keymember/3,foldl/3,reverse/1,last/1,sort/1]).
+-import(lists, [member/2,foldl/3,reverse/1,sort/1]).
 
 %% from_faces(FaceGbSet, We) -> VertexList
 %%  Convert a set of faces to a list of vertices.
@@ -345,16 +345,23 @@ min_distance_pairs_1(Faces0, Vs0, We0) ->
 	    end
     end.
 
-%% Don't go for position distance use the topological distance instead.
-%% Hopefully fixes this problem
-%  +__*_ *__*_+
-%   \       |  \
-%    \      |   \ 
-%     \      |   \
-%      \     |    \ 
-%       \    |     \
-%        \   |      \
-%         +--*--*--*-+
+%% Don't go for position distance - use the topological distance instead.
+%% Hopefully fixes this problem:
+%%     A  B  C
+%%  +__*_ *__*_+
+%%   \       |  \
+%%    \      |   \ 
+%%     \      |   \
+%%      \     |    \ 
+%%       \    |     \
+%%        \   |      \
+%%         +--*--*--*-+
+%%            D  E  F
+%%  What we do is to try connecting each selected vertex to the
+%%  next selected vertex near it. For instance, we might try the following
+%%  connections: A-B, B-C, C-D... The first two connections will be
+%%  discarded because the resulting new faces do not have defined
+%%  normals. The connection C-D will succeed.
 
 nearest_pair_smart(Face, AllVs, We) ->
     FaceVs = wings_face:vertices_ccw(Face, We),
@@ -362,7 +369,7 @@ nearest_pair_smart(Face, AllVs, We) ->
     Vs     = ordsets:intersection(Vs0, AllVs),
     nearest_pair_smart_1(FaceVs, Vs, Face, We, []).
 
-%% If we new that the intersection was stable this step wouldn't be needed.
+%% If we knew that the intersection was stable this step wouldn't be needed.
 nearest_pair_smart_1([V|Vs], Sel, Face, We, Acc) ->
     case ordsets:is_element(V, Sel) of
 	true ->
@@ -371,7 +378,7 @@ nearest_pair_smart_1([V|Vs], Sel, Face, We, Acc) ->
 	    nearest_pair_smart_1(Vs, Sel, Face, We, Acc)
     end;
 nearest_pair_smart_1([], _, Face, We, Acc=[Last|_]) when length(Acc) > 1 ->
-    connect_pairs([Last|lists:reverse(Acc)],Face,We);
+    connect_pairs([Last|reverse(Acc)],Face,We);
 nearest_pair_smart_1([], _, _, _, _) ->
     no.
     
@@ -417,10 +424,25 @@ try_connect({Va,Vb}, Face, We) ->
 
 try_connect_1(Va, Vb, Face, We0) ->
     {We,NewFace} = Res = force_connect(Va, Vb, Face, We0),
-    case wings_face:good_normal(Face, We) andalso
-	wings_face:good_normal(NewFace, We) of
-	true -> Res;
-	false -> no
+
+    %% It is crucial that we reject long thin faces, such as
+    %%
+    %%   A
+    %%   |
+    %%   |
+    %%   C
+    %%   |
+    %%   |
+    %%   B
+    %%
+    %% (where the edges are A-C, C-B, and B-A), since the
+    %% Connect command for vertices will try many combinations
+    %% of pair of vertices if the user has selected more than two
+    %% vertices.
+    case wings_face:is_thin(Face, We) orelse
+	wings_face:is_thin(NewFace, We) of
+	true -> no;
+	false -> Res
     end.
 
 force_connect(Vstart, Vend, Face, #we{es=Etab0,fs=Ftab0}=We0) ->
@@ -531,6 +553,10 @@ collect_outer_edges([Face|Fs], Faces, We, Acc0) ->
 		    outer_edge(Erec, Face, Faces, A)
 	    end, Acc0, Face, We),
     collect_outer_edges(Fs, Faces, We, Acc);
+
+collect_outer_edges([], _Faces, _We, []) ->
+    error;
+
 collect_outer_edges([], _Faces, _We, Acc) ->
     R = sofs:relation(Acc),
     F0 = sofs:relation_to_family(R),
@@ -557,7 +583,7 @@ order_edges(Va, [{Va,Vb}], Es0, Acc0) ->
 	    %% We have collected all outer vertices for one
 	    %% face region. We are done unless more edges remain
 	    %% (which is an error).
-	    case gb_sets:is_empty(Es0) of
+	    case gb_trees:is_empty(Es0) of
 		true -> Acc;
 		false -> error
 	    end;
