@@ -3,12 +3,12 @@
 %%
 %%     Window manager for Wings.
 %%
-%%  Copyright (c) 2002-2005 Bjorn Gustavsson
+%%  Copyright (c) 2002-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_wm.erl 217 2008-01-06 22:53:59Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_wm).
@@ -56,8 +56,7 @@
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
--import(lists, [map/2,foldl/3,last/1,sort/1,keysort/2,keysearch/3,
-		reverse/1,foreach/2,member/2]).
+-import(lists, [foldl/3,sort/1,keysort/2,reverse/1,foreach/2,member/2]).
 
 -define(Z_LOWEST_DYNAMIC, 10).
 
@@ -107,9 +106,6 @@ init() ->
     dirty_mode(back),
     resize_windows(W, H).
 
-desktop_event(got_focus) ->
-    dirty(),
-    keep;
 desktop_event(_) -> keep.
 
 message(Message) ->
@@ -529,7 +525,15 @@ get_and_dispatch() ->
     event_loop().
 
 dispatch_matching(Filter) ->
-    Evs = wings_io:get_matching_events(Filter),
+    Evs0 = wings_io:get_matching_events(Filter),
+
+    %% For historical reasons, because this module
+    %% enters all events using wings_io:putback_event_*/1,
+    %% we expect the matching events to be in reverse
+    %% order. Since wings_io:get_matching_events/1
+    %% now returns the events in same order as in the queue,
+    %% we must do a reverse here.
+    Evs = reverse(Evs0),
     foreach(fun dispatch_event/1, Evs).
 
 dispatch_event(#resize{w=W,h=H}) ->
@@ -566,13 +570,18 @@ dispatch_event(Event) ->
 update_focus(none) ->
     case erase(wm_focus) of
 	undefined -> ok;
-	OldActive -> do_dispatch(OldActive, lost_focus)
+	OldActive ->
+	    dirty(),
+	    do_dispatch(OldActive, lost_focus)
     end;
 update_focus(Active) ->
     case put(wm_focus, Active) of
-	undefined -> do_dispatch(Active, got_focus);
+	undefined ->
+	    dirty(),
+	    do_dispatch(Active, got_focus);
 	Active -> ok;
 	OldActive ->
+	    dirty(),
 	    do_dispatch(OldActive, lost_focus),
 	    do_dispatch(Active, got_focus)
     end.
@@ -1120,7 +1129,7 @@ message_redraw(Msg, Right) ->
 message_setup() ->
     wings_io:ortho_setup(none),
     {W,H} = win_size(),
-    wings_io:gradient_rect(0, 0, W, H, ?PANE_COLOR),
+    wings_io:gradient_rect(0, 0, W, H, wings_pref:get_value(info_line_bg)),
     gl:color3b(0, 0, 0),
     gl:translatef(10, H-5.375, 0),
     {W,H}.
