@@ -3,12 +3,12 @@
 %%
 %%     Implements toplevel windows.
 %%
-%%  Copyright (c) 2003-2004 Bjorn Gustavsson
+%%  Copyright (c) 2003-2009 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_wm_toplevel.erl 178 2008-01-01 10:11:00Z bjorng $
+%%     $Id$
 %%
 
 -module(wings_wm_toplevel).
@@ -23,7 +23,7 @@
 -include("wings.hrl").
 
 -import(lists, [reverse/1,keysearch/3,sort/1,foreach/2]).
--import(wings_util, [min/2,max/2]).
+-import(erlang, [min/2,max/2]).
 
 toplevel(Name, Title, Pos, Size, Flags, Op) ->
     wings_wm:new(Name, Pos, Size, Op),
@@ -219,9 +219,14 @@ ctrl_redraw(#ctrl{title=Title}) ->
     wings_io:ortho_setup(none),
     {W,_} = wings_wm:win_size(),
     TitleBarH = title_height(),
-    Pref = case {wings_wm:this(),wings_wm:actual_focus_window()} of
+    This = wings_wm:this(),
+    FocusWindow = wings_wm:actual_focus_window(),
+    Pref = case {This,FocusWindow} of
 	       {{_,Client},Client} -> title_active_color;
-	       {{_,Client},{_,Client}} -> title_active_color;
+	       % The Geometry Graph {object,geom} naming convention allows it to
+	       % match with the Geometry window, so we stop that particular match.
+	       {{_,Client},{WinElem,Client}} when WinElem =/= object ->
+	           title_active_color;
 	       {_,_} -> title_passive_color
 	   end,
     wings_io:blend(wings_pref:get_value(Pref),
@@ -504,7 +509,7 @@ resize_event(redraw, #rsz{color=Color}) ->
 resize_event(got_focus, _) ->
     wings_msg:button(?__(1,"Resize"), 
 		     ?__(2,"Resize, keeping current aspect ratio")),
-    wings_wm:dirty();
+    keep;
 resize_event(#mousebutton{button=1,state=?SDL_PRESSED},
 	     #rsz{state=moving,prev_focus=Focus}=Rst) ->
     wings_wm:grab_focus(Focus),
@@ -602,7 +607,7 @@ event(redraw, Ss) ->
     redraw(Ss);
 event(got_focus, _) ->
     wings_wm:message(""),
-    wings_wm:dirty();
+    keep;
 event({set_knob,Pos0,Prop0}, #ss{knob_pos=OldPos,knob_prop=OldProp}=Ss) ->
     case {max(0.0, min(1.0, Pos0)),max(0.0, min(1.0, Prop0))} of
 	{OldPos,OldProp} -> keep;
@@ -725,7 +730,7 @@ close_event(redraw) ->
     end;
 close_event(got_focus) ->
     wings_wm:message(?__(1,"Close this window")),
-    wings_wm:dirty();
+    keep;
 close_event(#mousebutton{button=1,state=?SDL_RELEASED}) ->
     {_,Client} = wings_wm:this(),
     wings_wm:send(Client, close),
@@ -763,8 +768,6 @@ get_menu_event(Mb) ->
 
 menubar_event(redraw, Mb) ->
     menubar_redraw(Mb);
-menubar_event(got_focus, _) ->
-    wings_wm:dirty();
 menubar_event({action,_}=Action, _) ->
     wings_wm:send(geom, Action);
 menubar_event(clear_menu_selection, Mb) ->
@@ -807,7 +810,7 @@ menubar_redraw(Mb) ->
     {menubar,Client} = wings_wm:this(),
     wings_io:ortho_setup(none),
     {W,H} = wings_wm:win_size(Client),
-    wings_io:border(0, 0, W-1, H-1, ?PANE_COLOR),
+    wings_io:border(0, 0, W-1, H-1, wings_pref:get_value(menu_bar_bg)),
     Menubar = wings_wm:get_menubar(Client),
     menubar_redraw_1(Menubar, Mb),
     get_menu_event(Mb#mb{bar=Menubar}).
